subroutine crm

!   time loop

use vars
use hbuffer
use microphysics
use sgs
use tracers
implicit none

integer icyc, nn
real tmp

do while(nstep.lt.nstop.and.nelapse.gt.0) 
        
  nstep = nstep + 1
  time = time + dt
  day = day0 + nstep*dt/86400.
  nelapse = nelapse - 1
!------------------------------------------------------------------
!  Check if the dynamical time step should be decreased 
!  to handle the cases when the flow being locally linearly unstable
!------------------------------------------------------------------

  ncycle = 1

  call kurant()

  total_water_before = total_water()
  total_water_evap = 0.
  total_water_prec = 0.
  total_water_ls = 0.

  do icyc=1,ncycle

     icycle = icyc
     dtn = dt/ncycle
     dt3(na) = dtn
     dtfactor = dtn/dt

     if(mod(nstep,nstatis).eq.0.and.icycle.eq.ncycle) then
        nstatsteps = nstatsteps + 1
        dostatis = .true.
        if(masterproc) print *,'Collecting statistics...'
     else
        dostatis = .false.
     endif

     !bloss:make special statistics flag for radiation,since it's only updated at icycle==1.
     dostatisrad = .false.
     if(mod(nstep,nstatis).eq.0.and.icycle.eq.1) dostatisrad = .true.

!---------------------------------------------
!  	the Adams-Bashforth scheme in time

     call abcoefs()

!---------------------------------------------
!  	initialize stuff: 

     call zero()

!------------------------------------------------------------

     total_water_ls =  total_water_ls - total_water()

!------------------------------------------------------------
!       Large-scale and surface forcing:

     call forcing()

!----------------------------------------------------------
!       Nadging to sounding:

     call nudging()

!----------------------------------------------------------
!   	spange-layer damping near the upper boundary:

!tmp = maxval(abs(dwdt(1:nx,1:ny,1:nzm,na)))
     call damping()
!print*,'damping:',rank,maxval(abs(dwdt(1:nx,1:ny,1:nzm,na))-tmp),maxloc(abs(dwdt(1:nx,1:ny,1:nzm,na))-tmp)
!----------------------------------------------------------

     total_water_ls =  total_water_ls + total_water()

!-----------------------------------------------------------
!	Radiation

      if(dolongwave.or.doshortwave) then 
        call radiation()     
      end if

!-----------------------------------------------------------
!       Buoyancy term:
     
     if(dodebug) call debug('before buo')
     call buoyancy()
     if(dodebug) call debug('after buo')

!----------------------------------------------------------
!     Update scalar boundaries after large-scale processes:

     call boundaries(2)

!-----------------------------------------------
!     surface fluxes:

     if(dosurface) call surface()

!-----------------------------------------------------------
!  SGS physics:

     if (dosgs) call sgs_proc()
!----------------------------------------------------------
!     Fill boundaries for SGS diagnostic fields:

     call boundaries(4)
!-----------------------------------------------
!       advection of momentum:

     call advect_mom()
     if(dodebug) call debug('after advect mom')

!----------------------------------------------------------
!	SGS effects on momentum:

     if(dosgs) call sgs_mom()
     if(dodebug) call debug('after sgs mom')

!-----------------------------------------------------------
!       Coriolis force:
     
     if(docoriolis) call coriolis()
     if(dodebug) call debug('after coriolis')
 
!---------------------------------------------------------
!       compute rhs of the Poisson equation and solve it for pressure. 

     call pressure()
     if(dodebug) call debug('after pressure')

!---------------------------------------------------------
!    update velocities.
!    compute velocity field at n+1/2 timestep needed for advection of scalars.
 
     call adams()

!---------------------------------------------------------
!      advection of scalars :

     call advect_all_scalars()

!-----------------------------------------------------------
!      sedimentation of cloud-ice :

    if(docloud) then
       call ice_fall()
    end if

!----------------------------------------------------------
!     Update boundaries for scalars to prepare for SGS effects:

     call boundaries(3)
   
!---------------------------------------------------------
!      SGS effects on scalars :

     if (dosgs) call sgs_scalars()

!-----------------------------------------------------------
!       Handle upper boundary for scalars

     if(doupperbound) call upperbound()

!-----------------------------------------------------------
!       Cloud condensation/evaporation and precipitation processes:

      if(docloud.or.dosmoke) call micro_proc()

!----------------------------------------------------------
!  Tracers' physics:

      call tracers_physics()

!-----------------------------------------------------------
!    Compute diagnostic fields:

      call diagnose()

!----------------------------------------------------------

! Rotate the dynamic tendency arrays for Adams-bashforth scheme:

      nn=na
      na=nc
      nc=nb
      nb=nn

   end do ! icycle	

!----------------------------------------------------------

  total_water_after = total_water()
          
!----------------------------------------------------------
!  collect statistics, write save-file, etc.

   call stepout()
  
!----------------------------------------------------------
!----------------------------------------------------------

end do ! main loop

!----------------------------------------------------------
!----------------------------------------------------------


end 
