
subroutine damping()

!  "Spange"-layer damping at the domain top region and around poles

use vars
use microphysics, only: micro_field, index_water_vapor
use params, only: dodamping, dolatlon, dodamping_poles, doterrain, dohyper, pi, dodamping_w
use terrain, only: terrau,terrav,terraw
use check_energy
implicit none

real, parameter :: nub=0.6  ! normalized height where damping begins at the equator
real nu, nustar, tau_max
real tau(ny,nzm) ! damping inverse time scale
real, parameter :: cuh_max=0.15  ! miximum courant number for damping u at poles
real, parameter :: cuz_max=0.15  ! miximum courant number for damping w 
real tauy(ny),umax(ny),wmax,ttau 
integer i, j, k, n_damp(nx,ny)

call t_startf ('damping')

call sumenergy(1,'damping')
!--------------------------------------------------------------------------------
!based on MetOffice Unified Model 6.0 and on Wood et al doi:10.1002/qj.2235
tau = 0.
tau_max = 1./(3.*dt)
do k=1,nzm
  nu = (zi(k)-zi(1))/(zi(nzm)-zi(1))
  do j=1,ny
   nustar = 1.+(nu-1.)*mu(j)
   if(nustar.gt.nub) then
     tau(j,k) = tau_max*sin(0.5*pi*(nustar-nub)/(1.-nub))**2
   end if
  end do
end do

if(dodamping) then

! damp vertical velocity at the top of domain 
  do k = 1,nzm
   do j = 1,ny
    if(tau(j,k).gt.0.) then
      do i=1,nx
        dwdt(i,j,k,na)= dwdt(i,j,k,na)- w(i,j,k)*tau(j,k)*terraw(i,j,k)
      end do! i 
    end if
   end do! j
  end do ! k

end if ! dodamping

! do k = 1,nzm
!    do j=1,ny
!     do i=1,nx
!       dvdt(i,j,k,na)= dvdt(i,j,k,na) - v(i,j,k)/3600.*terraw(i,j,k)
!     end do! i
!    end do! j
! end do ! k

!--------------------------------------------------------------------------------
if(dolatlon.and.dodamping_poles) then

! damp horizontal velocity around the poles and in stratosphere
  do j=1,ny
!    tauy(j) = 1./(3.*dtn)*(1.-mu(j)**2)**50
    umax(j) = cuh_max*dx*mu(j)/dt
  end do
  do k = 1,nzm
   do j=1,ny
       do i=1,nx
         dudt(i,j,k,na)= dudt(i,j,k,na)-  &
               (u(i,j,k)-min(umax(j),max(-umax(j),u(i,j,k))))*tau(j,k)*terrau(i,j,k)
       end do
   end do! j
  end do ! k

end if 

if(dodamping_w) then

  do j=1,ny
    tauy(j) = 1./(3.*dtn)
  end do
  do k = 1,nzm
   wmax = cuz_max*dz*adzw(k)/dt
   do j=1,ny
    if(tau(j,k).eq.0.) then
     do i=1,nx
        dwdt(i,j,k,na)= dwdt(i,j,k,na)-  &
               (w(i,j,k)-min(wmax,max(-wmax,w(i,j,k))))*tauy(j)*terraw(i,j,k)
     end do! i 
    end if
   end do! j
  end do ! k

end if

! try to filter 2-delta noise:

if(dohyper) call hyperdiff()

call sumenergy(-1,'damping')

call t_stopf('damping')




end subroutine damping
