
subroutine hyperdiff
	
!        apply hyperdiffusion (4th-derivative) to velocities
!        to control 2-delta waves. In horizontal direction only.
!        Code by Marat Khairoutdinov, 2016

use vars
use params, only: dowallx, dowally
use terrain
use check_energy
implicit none

real rdx4,rdy4

integer i,j,k,ja,jb,jd,je
real a(ny),b(ny),c(ny),d(ny),e(ny)
real rd,ddd,rc0,rc1

if(RUN2D) return

call sumenergy(1,'hyper')
!-----------------------------------------

rc0=1./1025. ! dimensionless dhyperdiff coef outside the poles
rc1=1./128. ! dimensionless dhyperdiff coef around the poles

rc0=rc0/dt
rc1=rc1/dt

do k=1,nzm
  do j=1,ny
   rd=max(rc0,rc1*(1.-mu(j)**2)**50)
   do i=1,nx
    ddd = min(terrau(i-2,j,k),terrau(i-1,j,k),terrau(i,j,k),terrau(i+1,j,k),terrau(i+2,j,k))
    dudt(i,j,k,na)=dudt(i,j,k,na)- &
             rd*ddd*(u(i-2,j,k)-4.*u(i-1,j,k)+6.*u(i,j,k)-4.*u(i+1,j,k)+u(i+2,j,k))
   end do
  end do
end do

do j=1,ny
 a(j) = imu(j)*imu(j-1)*muv(j)*muv(j-1)
 b(j) = imu(j)*muv(j)*(imu(j)*(muv(j)+muv(j+1))+imu(j-1)*(muv(j)+muv(j-1)))
 c(j) = imu(j)*(imu(j-1)*muv(j)*muv(j)+imu(j)*(muv(j)+muv(j+1))**2+imu(j+1)*muv(j+1)*muv(j+1))
 d(j) = imu(j)*muv(j+1)*(imu(j)*(muv(j)+muv(j+1))+imu(j+1)*(muv(j+1)+muv(j+2)))
 e(j) = imu(j)*imu(j+1)*muv(j+1)*muv(j+2)
end do

do k=1,nzm
  do j=1,ny
   ja=j-2*YES3D
   jb=j-YES3D
   jd=j+YES3D
   je=j+2*YES3D
   rd=max(rc0,rc1*(1.-mu(j)**2)**50)
   do i=1,nx
    ddd = min(terrau(i,ja,k),terrau(i,jb,k),terrau(i,j,k),terrau(i,jd,k),terrau(i,je,k))
    dudt(i,j,k,na)=dudt(i,j,k,na)- &
             rd*ddd*(a(j)*u(i,ja,k)-b(j)*u(i,jb,k)+c(j)*u(i,j,k)-d(j)*u(i,jd,k)+e(j)*u(i,je,k))
   end do
  end do
end do

do k=1,nzm
  do j=1,ny
   rd=max(rc0,rc1*(1.-muv(j)**2)**50)
   do i=1,nx
    ddd = min(terrav(i-2,j,k),terrav(i-1,j,k),terrav(i,j,k),terrav(i+1,j,k),terrav(i+2,j,k))
    dvdt(i,j,k,na)=dvdt(i,j,k,na)-rd*(v(i-2,j,k)-4.*v(i-1,j,k)+6.*v(i,j,k)-4.*v(i+1,j,k)+v(i+2,j,k))
   end do
  end do
end do

do j=1,ny
 a(j) = imuv(j)*imuv(j-1)*mu(j-1)*mu(j-2)
 b(j) = imuv(j)*mu(j-1)*(imuv(j)*(mu(j-1)+mu(j))+imuv(j-1)*(mu(j-1)+mu(j-2)))
 c(j) = imuv(j)*(imuv(j-1)*mu(j-1)*mu(j-1)+imuv(j)*(mu(j-1)+mu(j))**2+imuv(j+1)*mu(j)*mu(j))
 d(j) = imuv(j)*mu(j)*(imuv(j)*(mu(j-1)+mu(j))+imuv(j+1)*(mu(j)+mu(j+1)))
 e(j) = imuv(j)*imuv(j+1)*mu(j)*mu(j+1)
end do

do k=1,nzm
  do j=1,ny
   ja=j-2*YES3D
   jb=j-YES3D
   jd=j+YES3D
   je=j+2*YES3D
   rd=max(rc0,rc1*(1.-muv(j)**2)**50)
   do i=1,nx
    ddd = min(terrav(i,ja,k),terrav(i,jb,k),terrav(i,j,k),terrav(i,jd,k),terrav(i,je,k))
    dvdt(i,j,k,na)=dvdt(i,j,k,na)- &
             rd*ddd*(a(j)*v(i,ja,k)-b(j)*v(i,jb,k)+c(j)*v(i,j,k)-d(j)*v(i,jd,k)+e(j)*v(i,je,k))
   end do
  end do
end do

call sumenergy(-1,'hyper')

end subroutine hyperdiff
