module params

use grid, only: nzm

implicit none

!   Constants:

real, parameter :: cp = 1004.             ! Specific heat of air, J/kg/K
real, parameter :: ggr = 9.80616          ! Gravity acceleration, m/s2
real, parameter :: lcond = 2.5104e+06     ! Latent heat of condensation, J/kg
real, parameter :: lfus = 0.3336e+06      ! Latent heat of fusion, J/kg
real, parameter :: lsub = 2.8440e+06      ! Latent heat of sublimation, J/kg
real, parameter :: rv = 461.              ! Gas constant for water vapor, J/kg/K
real, parameter :: rgas = 287.            ! Gas constant for dry air, J/kg/K
real, parameter :: diffelq = 2.21e-05     ! Diffusivity of water vapor, m2/s
real, parameter :: therco = 2.40e-02      ! Thermal conductivity of air, J/m/s/K
real, parameter :: muelq = 1.717e-05      ! Dynamic viscosity of air

real, parameter :: fac_cond = lcond/cp 
real, parameter :: fac_fus = lfus/cp
real, parameter :: fac_sub = lsub/cp

real, parameter :: rad_earth = 6371229.   ! radius of the Earth (to make 40000 km circumference)
real, parameter :: pi = 3.141592653589793

!
! internally set parameters:

real   epsv     ! = (1-eps)/eps, where eps= Rv/Ra, or =0. if dosmoke=.true.
logical:: dosubsidence = .false.
real fcorz      ! Vertical Coriolis parameter
real coszrs
real salt_factor ! correction factor for water vapor saturation over sea-water
real:: sst_mean   ! mean sst (set only when dosstclimo is .true.)

integer:: ncycle_max = 8  ! maximum number of subcycling within dt
logical:: SLM = .false.

!----------------------------------------------
! Parameters set by PARAMETERS namelist:
! Initialized to default values.
!----------------------------------------------
logical:: dodebug = .false.

real:: ug = 0.        ! Velocity of the Domain's drift in x direction
real:: vg = 0.        ! Velocity of the Domain's drift in y direction
real:: fcor = -999.   ! Coriolis parameter	
real:: longitude0 = 0.    ! latitude of the domain's center 
real:: latitude0  = 0.    ! longitude of the domain's center 
real:: nxco2 = 1         ! factor to modify co2 concentration
real:: earth_factor = 1. ! radius of earth relative to Earth (real Earh/earth)

real(8):: tabs_s =0.    ! surface temperature,K
real:: delta_sst = 0.   ! amplitude of sin-pattern of sst about tabs_s (ocean_type=1)
real:: depth_slab_ocean = 2. ! thickness of the slab-ocean (m)
real:: Szero = 0.  ! mean ocean transport (W/m2)
real:: deltaS = 0. ! amplitude of linear variation of ocean transport (W/m2)
real:: timesimpleocean = 0. ! time to start simple ocean
real:: sst_climo =0.
real:: tau_ocean =0.

real::   fluxt0 =0.  ! surface sensible flux, Km/s
real::   fluxq0 =0.  ! surface latent flux, m/s
real::   tau0   =0.  ! surface stress, m2/s2
real::   z0     = 0.035 ! default roughness length (m)
real::   soil_wetness =1.! wetness coeff for soil (from 0 to 1.)
integer:: ocean_type =0 ! type of SST forcing
logical:: CEM =.false.    ! flag for Cloud Ensemble Model
logical:: LES =.false.    ! flag for Large-Eddy Simulation
logical:: OCEAN =.false.  ! flag indicating that surface is water
logical:: LAND =.false.   ! flag indicating that surface is land
logical:: ISLAND=.false.  ! flag to indicate that both ocean and land are present

logical:: doroundisland  = .false. ! to round islands 
real:: island_radius = 0. ! radius (in m) of a round island
! set island boundarues (default if not round island)
real:: island_x1 =  1.1  ! island boundary in units of fraction of domain in x
real:: island_x2 = -0.1 ! island boundary in units of fraction of domain in x
real:: island_y1 =  1.1  ! island boundary in units of fraction of domain in y
real:: island_y2 = -0.1 ! island boundary in units of fraction of domain in y

logical:: readinit = .false.   ! read 3D initial conditions from file
logical:: readlandmask = .false.   ! read landmask
logical:: readsst = .false. ! read sst from file
logical:: readterr = .false.   ! read terrain
logical:: reado3 = .false.   ! read ozone file
character(80):: initfile = ""
character(80):: landmaskfile = ""
character(80):: sstfile = ""
character(80):: terrainfile = ""
character(80):: o3file = ""

real:: n2ox = 0.  ! prescrined uniform n2o (g/g)
real:: ch4x = 0.  ! prescrined uniform ch4 (g/g)
real:: cfc11x = 0.  ! prescrined uniform cfc11 (g/g)
real:: cfc12x = 0.  ! prescrined uniform cfc12 (g/g)
logical:: sfc_flx_fxd =.false. ! surface sensible flux is fixed
logical:: sfc_tau_fxd =.false.! surface drag is fixed

real:: timelargescale =0. ! time to start large-scale forcing

! nudging boundaries (between z1 and z2, where z2 > z1): 
real:: nudging_uv_z1 =-1., nudging_uv_z2 = 1000000.
real:: nudging_t_z1 =-1., nudging_t_z2 = 1000000.
real:: nudging_q_z1 =-1., nudging_q_z2 = 1000000.
real:: tauls = 99999999.    ! nudging-to-large-scaler-profile time-scale
real:: tautqls = 99999999.! nudging-to-large-scaler-profile time-scale for scalars

real:: bufferzone = 0. ! fraction of the domain in x and y for nudging zone

logical:: dodamping = .false.
logical:: dodamping_poles = .false.
logical:: dodamping_w = .false.
logical:: dohyper = .false. ! apply 4th-order hyperviscosity to filter 2-delta waves
logical:: doupperbound = .false. 
logical:: docloud = .false. 
logical:: doprecip = .false.
logical:: dolongwave = .false. 
logical:: doshortwave = .false.
logical:: doradlat = .false.
logical:: doradlon = .false.
logical:: dosgs = .false.
logical:: docoriolis = .false. 
logical:: docoriolisz = .false. 
logical:: dofplane = .true.
logical:: dosurface = .false. 
logical:: dolargescale = .false. 
logical:: doradforcing = .false.
logical:: dosfcforcing = .false. 
logical:: doradsimple = .false. 
logical:: donudging_uv = .false. 
logical:: donudging_tq = .false.
logical:: donudging_t = .false. 
logical:: donudging_q = .false.
logical:: doensemble = .false. 
logical:: dowallx = .false. 
logical:: dowally = .false. 
logical:: docolumn = .false. 
logical:: docup = .false.
logical:: doperpetual = .false. 
logical:: doseasons = .false. 
logical:: doradhomo = .false. 
logical:: dosfchomo = .false.
logical:: dossthomo = .false. 
logical:: dodynamicocean = .false. 
logical:: dosstclimo = .false.
logical:: dosolarconstant = .false.
logical:: dotracers = .false. 
logical:: dosmoke = .false. 
logical:: notracegases = .false.
logical:: doseawater = .false.
logical:: doterrain = .false.
logical:: dobufferzone = .false.
logical:: dodynamicland = .false.
logical:: dodynamicsoil = .true.
logical:: dolatlon = .false. ! do lat/lon grid
logical:: doglobal = .false. ! global model. dx and dy are computed from lat of poles (latwalls)
                             ! domain is periodic in zonal direction
                             ! dolatlon and dowaly are set to .true. utomatically
logical:: dopointsource = .false.
integer:: pointsource_start_step = 1
logical:: dogrestart= .false. ! write or not general restart file
integer:: nstepgrestart = 0 ! timestep for writing general restart

! Specify solar constant and zenith angle for perpetual insolation.
! Based onn Tompkins and Graig (1998)
! Note that if doperpetual=.true. and dosolarconstant=.false.
! the insolation will be set to the daily-averaged value on day0.
real:: solar_constant = 685. ! solar constant (in W/m2)
real:: zenith_angle = 51.7   ! zenith angle (in degrees)

integer:: nensemble =0   ! the number of subensemble set of perturbations
integer:: perturb_type  = 0 ! type of initial noise in setperturb()
integer:: nens = 0          ! add this number to seed in setperturb to make ensembles
integer:: terrain_type = 0 ! type of the terrain initialization

! Initial bubble parameters. Activated when perturb_type = 2
  real:: bubble_x0 = 0.
  real:: bubble_y0 = 0.
  real:: bubble_z0 = 0.
  real:: bubble_radius_hor = 0.
  real:: bubble_radius_ver = 0.
  real:: bubble_dtemp = 0.
  real:: bubble_dq = 0.


end module params
