subroutine setgrid

! Initialize vertical grid

use vars	
use params
use terrain, only: setterrain, terra, terrau, terrav, terraw

implicit none
	
real latit, long, phi
integer i,j,it,jt,k, kmax, error
real(8) sums(nzm),sumu(nzm),sumv(nzm),sumw(nzm)
real(8) coef, coef1, buffer(nzm,4), buffer1(nzm,4)

error = 0

if(nrestart.eq.0) then

 
 if(doconstdz) then

    do k=2,nz
     zi(k)=dz*(k-1)
    end do
    do k=1,nzm
      adz(k) = (zi(k+1)-zi(k))/dz
      z(k) = 0.5*(zi(k+1)+zi(k))
    end do
    do k=2,nzm
      adzw(k) = (z(k)-z(k-1))/dz
    end do
    adzw(1) = 1.
    adzw(nz) = adzw(nzm)

 else

    open(8,file='./'//trim(case)//'/grd',status='old',form='formatted') 
    read(8,*) z(1)

    if(z(1).ne.0.) then 

! old (original) style of specifying grid levels in grd as height of scalar levels:

      do k=2,nzm     
        read(8,fmt=*,end=111) z(k)
        kmax=k
      end do
      goto 222
111   do k=kmax+1,nzm
       z(k)=z(k-1)+(z(k-1)-z(k-2))
      end do
222   continue
      close (8)
      dz = 0.5*(z(1)+z(2))
      do k=2,nzm
        adzw(k) = (z(k)-z(k-1))/dz
      end do
      adzw(1) = 1.
      adzw(nz) = adzw(nzm)
      adz(1) = 1.
      do k=2,nzm-1
        adz(k) = 0.5*(z(k+1)-z(k-1))/dz
      end do
      adz(nzm) = adzw(nzm)
      zi(1) = 0.
      do k=2,nz
        zi(k) = zi(k-1) + adz(k-1)*dz
      end do

    else 

! new style with grd specifying height of grid interfaces:

      zi(1) = z(1)
      do k=2,nz
        read(8,fmt=*,end=333) zi(k)
        kmax=k
      end do
      goto 444
333   do k=kmax+1,nz
       zi(k)=zi(k-1)+(zi(k-1)-zi(k-2))
      end do
444   continue
      close (8)
      dz = zi(2)-zi(1)
      do k=1,nzm
        adz(k) = (zi(k+1)-zi(k))/dz
        z(k) = 0.5*(zi(k+1)+zi(k))
      end do
      do k=2,nzm
        adzw(k) = (z(k)-z(k-1))/dz
      end do
      adzw(1) = 1.
      adzw(nz) = adzw(nzm)

    end if
 end if

end if

do k=1,nzm
  gamaz(k)=ggr/cp*z(k)
end do

! Initialize terrain

call setterrain()


if(z(nzm).gt.40000.) then
  if(masterproc) print*,'the domain top is too high: z(nzm) = ',z(nzm), '   exit...'
  call task_abort()
end if


if(dofplane) then

  if(fcor.eq.-999.) fcor = 4*pi/86400.*sin(latitude0*pi/180.)
  fcorz = 0.
  if(docoriolisz) fcorz =  sqrt(4.*(2*pi/(3600.*24.))**2-fcor**2)
  fcory(:) = fcor
  fcorzy(:) = fcorz
  longitude(:,:) = longitude0

else

 call task_rank_to_index(rank,it,jt)

 do j=0,ny
    if(dowally) then
     latit=latitude0+dy*(j+jt+0.5-(ny_gl+YES3D-1)/2-1)*2.5e-8*360.*earth_factor
    else
     latit=latitude0+dy*(j+jt-(ny_gl+YES3D-1)/2-1)*2.5e-8*360.*earth_factor
    end if
     fcory(j)= 4.*pi/86400.*sin(latit*pi/180.)*earth_factor
     fcorzy(j) = 0.
     if(j.ne.0.and.docoriolisz) fcorzy(j) = sqrt(4.*(2*pi/(3600.*24.))**2-fcory(j)**2)*earth_factor
 end do

end if ! dofplane

if(dolatlon) then 
 call task_rank_to_index(rank,it,jt)
 do j=dimy1_s,dimy2_s
     phi = latitude0+dy*(j+jt+0.5-(ny_gl+YES3D-1)/2-1)*2.5e-8*360.*earth_factor
     if(phi.ge.90.)  phi = latitude0+dy*(ny+jt+0.5-(ny_gl+YES3D-1)/2-1)*2.5e-8*360.*earth_factor
     if(phi.le.-90.) phi = latitude0+dy*(1+jt+0.5-(ny_gl+YES3D-1)/2-1)*2.5e-8*360.*earth_factor
     phi = pi/180.*phi
     mu(j) = cos(phi)
     tanr(j) = sin(phi)/cos(phi)/(rad_earth/earth_factor)
 end do
 do j=dimy1_v,dimy2_v
     muv(j) = 0.5*(mu(j)+mu(j-1)) ! this is nessesary for energy consistency of advection
 end do
 imu = 1./mu
 imuv = 1./muv
 do j=1,ny_gl
     mu_gl(j) = cos(pi/180.*(latitude0+dy*(j+0.5-(ny_gl+YES3D-1)/2-1)*2.5e-8*360.*earth_factor))
 end do
 do j=2,ny_gl
     muv_gl(j) = 0.5*(mu_gl(j)+mu_gl(j-1))
 end do
 muv_gl(1) = 2*muv_gl(2) - muv_gl(3)
 muv_gl(ny_gl+1)= 2*muv_gl(ny_gl) - muv_gl(ny_gl-1)
 imu_gl = 1./mu_gl
 imuv_gl = 1./muv_gl
 wgty(1:ny) = mu(1:ny)*ny_gl/sum(mu_gl(1:ny_gl))
end if


! compute weights for averaging:
sums = 0.
sumu = 0.
sumv = 0.
sumw = 0.
do k=1,nzm
 do j=1,ny
  do i=1,nx
    sums(k) = sums(k) + mu(j)*terra(i,j,k)
    sumu(k) = sumu(k) + mu(j)*terrau(i,j,k)
    sumv(k) = sumv(k) + muv(j)*terrav(i,j,k)
    sumw(k) = sumw(k) + mu(j)*terraw(i,j,k)
  end do
 end do
end do
if(dompi) then
  buffer(:,1) = sums(:)
  buffer(:,2) = sumu(:)
  buffer(:,3) = sumv(:)
  buffer(:,4) = sumw(:)
  call task_sum_real8(buffer,buffer1,nzm*4)
  sums(:) = buffer1(:,1)
  sumu(:) = buffer1(:,2)
  sumv(:) = buffer1(:,3)
  sumw(:) = buffer1(:,4)
end if
if(masterproc.and.(minval(sums).eq.0..or.minval(sumu).eq.0. &
                .or.minval(sumv).eq.0..or.minval(sumw).eq.0.)) then
    print*,'error in computing wgts in setgrid: terrain cannot be everywhere!'
    call task_abort()
end if
do k=1,nzm
 do j=1,ny
   wgt(j,k) = mu(j)*dble(nx_gl*ny_gl)/sums(k)
   wgtu(j,k) = mu(j)*dble(nx_gl*ny_gl)/sumu(k)
   wgtv(j,k) = muv(j)*dble(nx_gl*ny_gl)/sumv(k)
   wgtw(j,k) = mu(j)*dble(nx_gl*ny_gl)/sumw(k)
 end do
end do



if (dolatlon.or.doradlat) then
 call task_rank_to_index(rank,it,jt)
 do j=0,ny+1
     phi = latitude0+dy*(j+jt+0.5-(ny_gl+YES3D-1)/2-1)*2.5e-8*360.*earth_factor
     if(phi.le.-90.) phi = latitude0+dy*(1+jt+0.5-(ny_gl+YES3D-1)/2-1)*2.5e-8*360.*earth_factor
     if(phi.ge. 90.) phi = latitude0+dy*(ny+jt+0.5-(ny_gl+YES3D-1)/2-1)*2.5e-8*360.*earth_factor
     latitude(:,j) = phi
 end do
else
  latitude(:,:) = latitude0
end if

if (dolatlon.or.doradlon) then
 call task_rank_to_index(rank,it,jt)
 do i=1,nx
     longitude(i,:) = longitude0+dx/cos(latitude0*pi/180.)* &
                            (i+it-nx_gl/2-1)*2.5e-8*360.*earth_factor 
 end do
else
  longitude(:,:) = longitude0
end if

if(dolatlon.or.doradlat.or.doradlon) then
   call fminmax_print('lat(min/max):',latitude(:,1:ny),1,nx,1,ny,1)
   call fminmax_print('lon(min/max):',longitude,1,nx,1,ny,1)
end if

if(maxval(abs(latitude)).ge.90.) then
  print*,'rank=',rank,': error in setting latitude: max(abs(lat))=',maxval(abs(latitude))
  stop
end if
if(maxval(abs(longitude)).gt.360.) then
  print*,'rank=',rank,': error in setting longitude: max(lon)=',maxval(abs(longitude))
  stop
end if


end
