
subroutine setperturb

!  Random noise

use vars
use params
use microphysics, only: micro_field, index_water_vapor
use sgs, only: setperturb_sgs
use terrain, only: k_terra

implicit none

integer i,j,k,ptype,it,jt
real rrr,ranf_
real xxx,yyy,zzz
real nu, nuv, phi, lamda

call ranset_(3*rank+nens)

ptype = perturb_type

call setperturb_sgs(ptype)  ! set sgs fields

select case (ptype)

  case(-1)

! do nothing

  case(0)

     do k=1,nzm
      do j=1,ny
       do i=1,nx
         rrr=1.-2.*ranf_()
         if(k.ge.k_terra(i,j).and.k.le.k_terra(i,j)+5) then
            t(i,j,k)=t(i,j,k)+0.05*rrr*dx/1000.
         endif
       end do
      end do
     end do

  case(1)

     do k=1,nzm
      do j=1,ny
       do i=1,nx
         rrr=1.-2.*ranf_()
         if(q0(k).gt.6.e-3) then
            t(i,j,k)=t(i,j,k)+0.1*rrr
         endif
       end do
      end do
     end do

  case(2) ! warm bubble

     if(masterproc) then
       print*, 'initialize with warm bubble:'
       print*, 'bubble_x0=',bubble_x0
       print*, 'bubble_y0=',bubble_y0
       print*, 'bubble_z0=',bubble_z0
       print*, 'bubble_radius_hor=',bubble_radius_hor
       print*, 'bubble_radius_ver=',bubble_radius_ver
       print*, 'bubble_dtemp=',bubble_dtemp
       print*, 'bubble_dq=',bubble_dq
     end if

     call task_rank_to_index(rank,it,jt)
     do k=1,nzm
       zzz = z(k)
       do j=1,ny
         yyy = dy*(j+jt-1)
         do i=1,nx
          xxx = dx*(i+it-1)
           if((xxx-bubble_x0)**2+YES3D*(yyy-bubble_y0)**2.lt.bubble_radius_hor**2 &
            .and.(zzz-bubble_z0)**2.lt.bubble_radius_ver**2) then
              rrr = cos(pi/2.*(xxx-bubble_x0)/bubble_radius_hor)**2 &
               *cos(pi/2.*(yyy-bubble_y0)/bubble_radius_hor)**2 &
               *cos(pi/2.*(zzz-bubble_z0)/bubble_radius_ver)**2
              t(i,j,k) = t(i,j,k) + bubble_dtemp*rrr
              if(mod(rank,2).eq.0.and.i.eq.(nx/2).and.k.eq.5.and.(j.eq.1.or.j.eq.ny)) print*,rank,j,t(i,j,k)
              micro_field(i,j,k,index_water_vapor) = &
                  micro_field(i,j,k,index_water_vapor) + bubble_dq*rrr
           end if
         end do
       end do
     end do

  case(3)   ! gcss wg1 smoke-cloud case

     do k=1,nzm
      do j=1,ny
       do i=1,nx
         rrr=1.-2.*ranf_()
         if(q0(k).gt.0.5e-3) then
            t(i,j,k)=t(i,j,k)+0.1*rrr
         endif
       end do
      end do
     end do

  case(4)  ! gcss wg1 arm case

     do k=1,nzm
      do j=1,ny
       do i=1,nx
         rrr=1.-2.*ranf_()
         if(z(k).le.200.) then
            t(i,j,k)=t(i,j,k)+0.1*rrr*(1.-z(k)/200.)
         endif
       end do
      end do
     end do

  case(5)  ! gcss wg1 BOMEX case

     do k=1,nzm
      do j=1,ny
       do i=1,nx
         rrr=1.-2.*ranf_()
         if(z(k).le.1600.) then
            t(i,j,k)=t(i,j,k)+0.1*rrr
            micro_field(i,j,k,index_water_vapor)= &
                      micro_field(i,j,k,index_water_vapor)+0.025e-3*rrr
         endif
       end do
      end do
     end do

  case(6)  ! GCSS Lagragngian ASTEX


     do k=1,nzm
      do j=1,ny
       do i=1,nx
         rrr=1.-2.*ranf_()
         if(q0(k).gt.6.e-3) then
            t(i,j,k)=t(i,j,k)+0.1*rrr
            micro_field(i,j,k,index_water_vapor)= &
                      micro_field(i,j,k,index_water_vapor)+2.5e-5*rrr
         endif
       end do
      end do
     end do

  case(7)  ! Wind on sphere

     call task_rank_to_index(rank,it,jt)
     do k=1,nzm
      do j=1,ny
       do i=1,nx
           t(i,j,k) = t(i,j,k) +100.*sin(pi*dble(j+jt-1)/dble(ny_gl-1))*sin(pi*z(k)/z(nzm))
       end do
      end do
     end do

  case(8)  ! Jablonowski & Williamson (2006) hydrostatic test

     do k=1,nzm
      nu = pres(k)/1000.
      nuv = pi/2.*(nu-0.252)
      if(nu.ge.0.2) then
         t(:,:,k) = 288.*nu**(287.*0.005/9.80616)
      else
         t(:,:,k) = 288.*nu**(287.*0.005/9.80616)+4.8e5*(0.2-nu)**5
      end if 
      t(:,:,k) = t(:,:,k)+gamaz(k)
      do j=0,ny+1
       do i=1,nx
           phi = latitude(i,j)*pi/180.
           t(i,j,k) = t(i,j,k) +0.75*nu*pi*35./287.*sin(nuv)*cos(nuv)**0.5 &
             *((-2.*sin(phi)**6*(cos(phi)**2+1./3.)+10./63.)*2.*35.*cos(nuv)**1.5 &
               +(8./5.*cos(phi)**3*(sin(phi)**2+2./3.)-pi/4.)*rad_earth*7.29212e-5)
           u(i,j,k) = 35.*cos(nuv)**1.5*sin(2.*phi)**2
       end do
      end do
     end do
     micro_field(:,:,:,:) = 0.
     call diagnose()
     bet(:) = ggr/tabs0(:)

     case(81) 
! baroclinic test perturbation:
     do k=1,nzm
      do j=1,ny
       do i=1,nx
         lamda = longitude(i,j)*pi/180.
         phi = latitude(i,j)*pi/180.
         u(i,j,k) = u(i,j,k) + &
          exp(-100.*acos(sin(2.*pi/9.)*sin(phi)+cos(2.*pi/9.)*cos(phi)*cos(lamda-pi/9.+pi))**2)
       end do
      end do
     end do

  case(9)  ! point source

     call task_rank_to_index(rank,it,jt)
     do j=1,ny
      do i=1,nx
        if(i+it.ge.nx_gl/10-2.and.i+it.le.nx_gl/10+2.and. &
           j+jt.ge.ny_gl/2-ny_gl/4.and.j+jt.le.ny_gl/2+ny_gl/4) &
          micro_field(i,j,1:3,index_water_vapor)=micro_field(i,j,1:3,index_water_vapor)+15.e-3
      end do
     end do
     print*,'>>',rank,minval(micro_field),maxval(micro_field)


  case default

       if(masterproc) print*,'perturb_type is not defined in setperturb(). Exitting...'
       call task_abort()

end select


end

