module terrain

!incorporation of terrain into SAM
!Author: Marat Khairoutdinov, Nov. 2011
!--------------------------------------------------------------------

use grid
use vars, only: elevation, elevationg, landmask
use params, only: terrainfile, terrain_type, doterrain, pi,dowallx,dowally

implicit none

! terrain masks: inside terrain? (1 no, 0 yes, in between - kind of)
real terrau   (dimx1_u:dimx2_u, dimy1_u:dimy2_u, nzm) 
real terrav   (dimx1_v:dimx2_v, dimy1_v:dimy2_v, nzm) 
real terraw   (dimx1_w:dimx2_w, dimy1_w:dimy2_w, nz ) 
real terra    (dimx1_s:dimx2_s, dimy1_s:dimy2_s, nzm) 

integer terr_index(nx, ny) ! vertical index
integer k_terra(dimx1_s:dimx2_s, dimy1_s:dimy2_s) ! vertical grid index of terrain top
integer k_terrau(dimx1_u:dimx2_u, dimy1_u:dimy2_u) 
integer k_terrav(dimx1_v:dimx2_v, dimy1_v:dimy2_v) 

real(4) mask(nx_gl,ny_gl)
real, external :: ranf_ 

contains

subroutine setterrain

!  Random noise

use vars, only: u,v,w,t
implicit none

integer i,j,k,m,it,jt,nx1,ny1,kb,kc,iter
real xxx,yyy,zzz, aaa, bbb, ccc
real(4) hz(nx_gl,ny_gl)
real terr(0:nx_gl+1, 0:ny_gl+1)
real tmp(1:nx,1:ny,nzm)

terra = 1.
terrau = 1.
terrav = 1.
terraw = 1.
k_terra = 1
k_terrau = 1
k_terrav = 1

if(.not.doterrain) then
  elevation = 0.
  return
end if

if(nrestart.eq.0) then

  call task_rank_to_index(rank,it,jt)

  select case (terrain_type)

    case(0)

    case(1) ! bell-shaped mountain

  ! Bouldar case:
       aaa = 10000. ! half-width
       bbb = 2000. ! height
       do j=1,ny
         yyy = dy*(j+jt-1)
         do i=1,nx
          xxx = dx*(i+it-1)
           elevation(i,j) = bbb*aaa**2/(aaa**2+(xxx-0.5*nx_gl*dx)**2+&
                      YES3D*(yyy-0.5*ny_gl*dy)**2)
           end do
         end do
  
    case(2) ! Building
  
       aaa=0.2*ny_gl*dx
       bbb=2000.
       elevation = 0.
       do j=1,ny
        yyy = dy*(j+jt-1)
         do i=1,nx
          xxx = dx*(i+it-1)
           if(abs(xxx-0.5*(nx_gl+0.5)*dx).le.aaa.and. &
              abs(yyy-0.5*(ny_gl+0.5)*dx).le.aaa ) then
                elevation(i,j) = bbb
           end if
         end do
       end do
  
    case(3) ! kink
  
       do j=1,ny
         yyy = dy*(j+jt-1)
         do i=1,nx
          xxx = dx*(i+it-1)
           elevation(i,j)=max(0.,0.95*zi(2)*(1.-abs(xxx-0.5*nx_gl*dx)/(30.*dx)))
         end do
       end do
 

    case(4) ! squared-cosine hill
 
       aaa = 0.2*nx_gl*dx ! width
       bbb = 4000. ! height
       do j=1,ny
         yyy = dy*(j+jt-1)
         do i=1,nx
          xxx = dx*(i+it-1)
          zzz = min(0.25,((xxx-(0.5*nx_gl+0.5)*dx)/aaa)**2+(YES3D*(yyy-(0.5*ny_gl+0.5)*dy)/aaa)**2)
          elevation(i,j) = bbb*cos(pi*sqrt(zzz))**2
         end do
       end do
  
    case(5) ! squared-cosine valley

       aaa = 500. ! half-width
       bbb = 200. ! depth
       do j=1,ny
         yyy = dy*(j+jt-1)
         do i=1,nx
          xxx = dx*(i+it-1)
          zzz = ((xxx-0.5*nx_gl*dx)/aaa)**2+(YES3D*(yyy-0.5*ny_gl*dy)/aaa)**2
          elevation(i,j) = max(0.,bbb-bbb*cos(pi*sqrt(zzz))**2)
        end do
       end do

     case(6) ! round island
       
       elevation = 0.
       where(landmask.eq.1) elevation = z(3)

     case(7) ! random terrain

       elevation = 0.
       call ranset_(5*rank)
       do j=1,ny
        do i=1,nx
         k=ranf_()*4
         elevation(i,j) = zi(k)
        end do
       end do
       print*,'>>>',rank,minval(elevation),maxval(elevation)

    case default

       if(masterproc) print*,'terrain_type ',terrain_type, &
                     ' is not defined in setperturb(). Exitting...'
       call task_abort()

  end select

end if ! nrestart.eq.0

do k=1,nzm
  do j=1,ny
    do i=1,nx
        if(elevation(i,j).ge.zi(k+1)) then 
         terra(i,j,k) = 0.
        end if
    end do
  end do
end do

tmp(1:nx,1:ny,1:nzm) = t(1:nx,1:ny,1:nzm)
t = terra
if(dompi) then
 call task_exchange(t,dimx1_s,dimx2_s,dimy1_s,dimy2_s,nzm,3+NADVS,3+NADVS,3+NADVS,3+NADVS,4)
else
 call bound_exchange(t,dimx1_s,dimx2_s,dimy1_s,dimy2_s,nzm,3+NADVS,3+NADVS,3+NADVS,3+NADVS,4)
endif
terra = t
t(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)

if(dowallx) then
  if(mod(rank,nsubdomains_x).eq.0) then
    do k=1,nzm
     do j=dimy1_s,dimy2_s
       do i=dimx1_s,0
         terra(i,j,k) = terra(1,j,k)
       end do
     end do
    end do
  end if
  if(mod(rank,nsubdomains_x).eq.nsubdomains_x-1) then
    do k=1,nzm
     do j=dimy1_s,dimy2_s
       do i=nx+1,dimx2_s
         terra(i,j,k) = terra(nx,j,k)
       end do
     end do
    end do
  end if
end if

if(dowally) then
  if(rank.lt.nsubdomains_x) then
    do k=1,nzm
     do j=dimy1_s,0
       do i=dimx1_s,dimx2_s
         terra(i,j,k) = terra(i,1,k)
       end do
     end do
    end do
  end if
  if(rank.gt.nsubdomains-nsubdomains_x-1) then
    do k=1,nzm
     do j=ny+1,dimy2_s
       do i=dimx1_s,dimx2_s
         terra(i,j,k) = terra(i,ny,k)
       end do
     end do
    end do
  end if
end if

do k=1,nzm
  do j=1,ny
    do i=1,nx
        if(terra(i,j,k).lt.1.) then 
         elevationg(i,j) = z(k)
        end if
    end do
  end do
end do

do k=1,nzm
  do j=dimy1_s,dimy2_s
    do i=dimx1_s,dimx2_s
       if(terra(i,j,k).lt.1.) k_terra(i,j) = k+1
    end do
  end do
end do

do k=1,nzm
  do j=1,ny
    do i=1,nx
        if(terra(i,j,k).lt.1) then
         terraw(i,j,k) = 0
         terraw(i,j,k+1) = 0
        end if
    end do
  end do
end do


do k=1,nzm
  do j=1,ny
    do i=1,nx
       terrau(i,j,k) = min(terra(i-1,j,k),terra(i,j,k))
       terrav(i,j,k) = min(terra(i,j-YES3D,k),terra(i,j,k))
    end do
  end do
end do

call task_barrier()

if(dompi) then
 tmp(1:nx,1:ny,1:nzm) = u(1:nx,1:ny,1:nzm)
 u=terrau
 call task_exchange(u,dimx1_u,dimx2_u,dimy1_u,dimy2_u,nzm,2,3,2+NADV,2+NADV,1)
 terrau=u
 u(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)
 tmp(1:nx,1:ny,1:nzm) = v(1:nx,1:ny,1:nzm)
 v=terrav
 call task_exchange(v,dimx1_v,dimx2_v,dimy1_v,dimy2_v,nzm,2+NADV,2+NADV,2,3,2)
 terrav=v
 v(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)
 tmp(1:nx,1:ny,1:nzm) = w(1:nx,1:ny,1:nzm)
 w=terraw
 call task_exchange(w,dimx1_w,dimx2_w,dimy1_w,dimy2_w,nz,2+NADV,2+NADV,2+NADV,2+NADV,3)
 terraw=w
 w(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)
 w(1:nx,1:ny,nz) = 0.
else
 tmp(1:nx,1:ny,1:nzm) = u(1:nx,1:ny,1:nzm)
 u=terrau
 call bound_exchange(u,dimx1_u,dimx2_u,dimy1_u,dimy2_u,nzm,2,3,2+NADV,2+NADV,1)
 terrau=u
 u(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)
 tmp(1:nx,1:ny,1:nzm) = v(1:nx,1:ny,1:nzm)
 v=terrav
 call bound_exchange(v,dimx1_v,dimx2_v,dimy1_v,dimy2_v,nzm,2+NADV,2+NADV,2,3,2)
 terrav=v
 v(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)
 tmp(1:nx,1:ny,1:nzm) = w(1:nx,1:ny,1:nzm)
 w=terraw
 call bound_exchange(w,dimx1_w,dimx2_w,dimy1_w,dimy2_w,nz,2+NADV,2+NADV,2+NADV,2+NADV,3)
 terraw=w
 w(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)
endif

call task_barrier()

call boundaries(1)
call boundaries(2)

do k=1,nzm,1
  do j=dimy1_u,dimy2_u
    do i=dimx1_u,dimx2_u
       if(terrau(i,j,k).lt.1.) k_terrau(i,j) = k+1
    end do
  end do
end do
do k=1,nzm,1
  do j=dimy1_v,dimy2_v
    do i=dimx1_v,dimx2_v
       if(terrav(i,j,k).lt.1.) k_terrav(i,j) = k+1
    end do
  end do
end do

end subroutine setterrain

end module terrain
