integer function lenstr (string)
      
! returns string's length ignoring the rightmost blank and null characters

implicit none
character *(*) string
integer k
lenstr = 0
do k = 1,len(string)
 if (string(k:k).ne.' '.and.string(k:k).ne.char(0)) then
     lenstr = lenstr+1
 end if
end do
111  return
end



subroutine averageXY(f,dimx1,dimx2,dimy1,dimy2,dimz,fm)

! average scalar field
use grid, only: nx,ny,nzm,wgt
use terrain, only: terra
implicit none
integer dimx1, dimx2, dimy1, dimy2, dimz
real f(dimx1:dimx2, dimy1:dimy2, dimz),fm(nzm) 
real(8) ff,factor
integer i,j,k	
factor = 1./dble(nx*ny)
do k =1,nzm
 ff = 0.
 do j =1,ny
  do i =1,nx
    ff = ff + f(i,j,k)*wgt(j,k)*terra(i,j,k)
  end do
 end do
 ff = ff*factor
 fm(k) = real(ff)
end do 
end


	
subroutine averageXY_MPI(f,dimx1,dimx2,dimy1,dimy2,dimz,fm)

! average scalar field
use grid, only: dompi,nsubdomains,nx,ny,nzm,wgt
use terrain, only: terra
implicit none
integer dimx1, dimx2, dimy1, dimy2, dimz
real f(dimx1:dimx2, dimy1:dimy2, dimz),fm(nzm)
real(8) fm1(nzm),fm2(nzm),factor
integer i,j,k
factor = 1./dble(nx*ny)
do k =1,nzm
 fm1(k) = 0.
 do j =1,ny
  do i =1,nx
    fm1(k) = fm1(k) + f(i,j,k)*wgt(j,k)*terra(i,j,k)
  end do
 end do
 fm1(k) = fm1(k) * factor
end do
if(dompi) then
 do k =1,nzm
   fm2(k) = fm1(k)
 end do
 call task_sum_real8(fm2,fm1,nzm)
 do k=1,nzm
  fm(k)=real(fm1(k)/dble(nsubdomains))
 end do
else
 do k=1,nzm
  fm(k)=real(fm1(k))
 end do
endif
end


		
	
	
subroutine fminmax_print(name,f,dimx1,dimx2,dimy1,dimy2,dimz)

use grid
implicit none
integer dimx1, dimx2, dimy1, dimy2, dimz
real f(dimx1:dimx2, dimy1:dimy2, dimz),fmn(nz),fmx(nz)
character *(*) name
real fmin(1),fmax(1),fff(1),fsum(1)
integer i,j,k
	
do k=1,dimz
 if(dimx2.eq.1.and.dimy2.eq.1) then
   fmn(k) = f(1,1,k)	
   fmx(k) = f(1,1,k)	
 else
   fmn(k) = 1.e30
   fmx(k) =-1.e30
   do j=1,ny
    do i=1,nx
     fmn(k) = min(fmn(k),f(i,j,k))
     fmx(k) = max(fmx(k),f(i,j,k))
    end do
   enddo
 end if
enddo
fmin(1) = 1.e30
fmax(1) =-1.e30
do k=1,dimz
 fmin(1) = min(fmin(1),fmn(k))
 fmax(1) = max(fmax(1),fmx(k))
end do
	
if(dompi) then
  fff(1)=fmax(1)
  call task_max_real(fff(1),fmax(1),1)
  fff(1)=fmin(1)
  call task_min_real(fff(1),fmin(1),1)
  call task_min_real(fff(1),fmin(1),1)
!  call task_sum_real(sum(f),fsum,1)
else
  fsum = sum(f)
end if
if(masterproc) print *,name,fmin,fmax !,fsum
end

subroutine fminmax1_print(name,f,n1,n2,n3)

use grid
implicit none
integer n1,n2,n3
real f(n1,n2,n3)
character *(*) name
real fmin(1),fmax(1),fff(1)

fmin(1) = minval(f)
fmax(1) = maxval(f)

if(dompi) then
  fff(1)=fmax(1)
  call task_max_real(fff(1),fmax(1),1)
  fff(1)=fmin(1)
  call task_min_real(fff(1),fmin(1),1)
end if
if(masterproc) print *,name,fmin,fmax
end


	
subroutine setvalue(f,n,f0)
implicit none
integer n
real f(n), f0
integer k
do k=1,n
 f(k)=f0
end do
end

! determine number of byte in a record in direct access files (can be anything, from 1 to 8):
! can't assume 1 as it is compiler and computer dependent
integer function bytes_in_rec()
implicit none
character*8 str
integer n, err
open(1,status ='scratch',access ='direct',recl=1)
do n = 1,8
 write(1,rec=1,iostat=err) str(1:n)
 if (err.ne.0) exit
 bytes_in_rec = n
enddo
close(1,status='delete')
end

!----------------------------------------
! average a global 3D array alone x direction
! with the result known to each subdomain

subroutine mean_x_3D(in, out)

use grid
implicit none
real, intent(in) ::  in(nx,ny,nzm)
real, intent(out) :: out(ny,nzm)
real buf(ny,nzm,max(1,nsubdomains_x-1))
integer i,j,k,m
integer n_in, count, reqs_in(max(1,nsubdomains_x-1)), rnk, tag
logical flag(nsubdomains_x)

! average locally first:
do k=1,nzm
 do j=1,ny
  out(j,k) = sum(in(1:nx,j,k))
 end do
end do
out(:,:)=out(:,:)/dble(nx)

if(dompi) then

 if(mod(rank,nsubdomains_x).ne.0) then
   call task_bsend_float(rank-mod(rank,nsubdomains_x),out(1:ny,1:nzm),ny*nzm, 31)
   call task_breceive_float(out(1:ny,1:nzm),ny*nzm,rnk,tag)
 else
   n_in = 0
   do i=1,nsubdomains_x-1
     n_in = n_in + 1
     call task_receive_float(buf(1:ny,1:nzm,n_in),ny*nzm,reqs_in(n_in))
     flag(n_in) = .false.
   end do
   count = n_in
   do while (count .gt. 0)
    do m = 1,n_in
     if(.not.flag(m)) then
         call task_test(reqs_in(m), flag(m), rnk, tag)
         if(flag(m)) then
           count=count-1
           out(1:ny,1:nzm) = out(1:ny,1:nzm) + buf(1:ny,1:nzm,m)
         endif
     endif
    end do
   end do
   out = out/dble(nsubdomains_x)
   do i=1,nsubdomains_x-1
     call task_send_float(rank+i,out(1:ny,1:nzm),ny*nzm,31,rnk)
   end do
 end if

end if

call task_barrier()

end
!----------------------------------------
! average a global 2D array (x-y) alone x direction
! with the result known to each subdomain

subroutine mean_x_2D(in, out)

use grid
implicit none
real, intent(in) ::  in(nx,ny)
real, intent(out) :: out(ny)
real buf(ny,max(1,nsubdomains_x-1))
integer i,j,k,m
integer n_in, count, reqs_in(max(1,nsubdomains_x-1)), rnk, tag
logical flag(nsubdomains_x)

! average locally first:
do j=1,ny
 out(j) = sum(in(1:nx,j))/dble(nx)
end do

if(dompi) then

 if(mod(rank,nsubdomains_x).ne.0) then
   call task_bsend_float(rank-mod(rank,nsubdomains_x),out(1:ny),ny, 31)
   call task_breceive_float(out(1:ny),ny,rnk,tag)
 else
   n_in = 0
   do i=1,nsubdomains_x-1
     n_in = n_in + 1
     call task_receive_float(buf(1:ny,n_in),ny,reqs_in(n_in))
     flag(n_in) = .false.
   end do
   count = n_in
   do while (count .gt. 0)
    do m = 1,n_in
     if(.not.flag(m)) then
         call task_test(reqs_in(m), flag(m), rnk, tag)
         if(flag(m)) then
           count=count-1
           out(1:ny) = out(1:ny) + buf(1:ny,m)
         endif
     endif
    end do
   end do
   out = out/dble(nsubdomains_x)
   do i=1,nsubdomains_x-1
     call task_send_float(rank+i,out(1:ny),ny,31,rnk)
   end do
 end if

end if

call task_barrier()
end


subroutine readsurface(filename,field,dayobs)

! read surface data 

use grid
implicit none
character*(*), intent(in) ::  filename
real, intent(out) ::  field(nx,ny,2)   ! two time saples if available
real, intent(out) :: dayobs(2) ! days corresponding to field samples

integer i,nn,it,jt,nobs,nx1,ny1
real(4) fld(1:nx_gl,1:ny_gl)
real(4), allocatable :: days(:)
real coef

call task_rank_to_index(rank,it,jt)
if(masterproc) print*,'reading data from ',trim(filename)
open(11,file=trim(filename),status='old',form='unformatted')
read(11) nobs
read(11) nx1
read(11) ny1
if(nx1.ne.nx_gl.or.ny1.ne.ny_gl) then
  if(masterproc) print*,'dimensions of domain in file '//trim(filename)//' is' // &
              'different from model domain size: nx=',nx1, &
              'ny=',ny1,'  Quitting...'
  call task_abort()
end if
allocate (days(nobs))
read(11) days(1:nobs)
nn=1
do i=1,nobs-1
 if(day.gt.days(i)) then
   nn=i
 endif
end do
do i=1,nn-1
   read(11)
end do
field = 0.
read(11) fld(1:nx_gl,1:ny_gl)
call task_rank_to_index(rank,it,jt)
field(1:nx,1:ny,1) = fld(1+it:nx+it,1+jt:ny+jt)
dayobs(1) = days(nn)
dayobs(2) = 0.
if(nobs.gt.1) then
 read(11) fld(1:nx_gl,1:ny_gl)
 field(1:nx,1:ny,2) = fld(1+it:nx+it,1+jt:ny+jt)
 dayobs(2) = days(nn+1)
end if
close (11)
if(masterproc) print*,'min/max:',minval(fld),maxval(fld)
if(nobs.gt.1.and.masterproc) print*,'days',days

end


subroutine readlevels(filename,field)

! read layered data

use grid
implicit none
character*(*), intent(in) ::  filename
real, intent(out) ::  field(nx,ny,nzm)   ! output field

integer i,k,nn,nobs,nx1,ny1,nz1
real fld(nx,ny,nzm)
real(4), allocatable :: days(:)
real coef, dayy

if(masterproc) print*,'reading data from ',trim(filename)
open(11,file=trim(filename),status='old',form='unformatted')
read(11) nobs
read(11) nx1
read(11) ny1
read(11) nz1
if(masterproc)print*,'nx ny nz:',nx1,ny1,nz1
allocate (days(nobs))
read(11) days(1:nobs)
nn=1
do i=1,nobs-1
 if(day.gt.days(i)) then
   nn=i
 endif
end do
do i=1,nn-1
 read(11)
 do k=1,nz1
   read(11)
 end do
end do
call restart_read_field3D(11,field,nx1,ny1,nzm)
dayy = day
if(nobs.gt.1) then
 call restart_read_field3D(11,fld,nx1,ny1,nzm)
 do while(dayy.gt.365.)
  dayy = dayy-365.
 end do
 if(masterproc) print*,'interpolating: day=',dayy,'day1=',days(nn),'day2=',days(nn+1)
 coef=(dayy-days(nn))/(days(nn+1)-days(nn))
 field(:,:,:) = field(:,:,:)+coef*(fld(:,:,:)-field(:,:,:))
end if
close (11)
if(masterproc) print*,'min/max:',minval(field),maxval(field)

end


subroutine print_date_from_day()
! return a string contain date (day and month) and UTC time.
use grid, only: day,year0
implicit none
integer, parameter :: days_in_mon(12) = (/31,28,31,30,31,30,31,31,30,31,30,31/)
character(3), parameter :: mon(12) =  &
      (/"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"/)
integer i, iday, ihour, imins
real dd,ddd
dd=mod(int(day),365)+day-int(day)
ddd=0.
do i=1,12
 ddd=ddd+days_in_mon(i)
 if(ddd.ge.dd) then
   ddd=ddd-days_in_mon(i)
   iday = dd-ddd
   ihour = (dd-int(dd))*24
   imins = ((dd-int(dd))*24-ihour)*60.
   exit
 end if
end do
if(year0.eq.0) then
  write(*,'(i3,a4,i4,a1,i2)') iday,mon(i),ihour,':',imins
else
  write(*,'(i3,a4,i4,a1,i2,i6)') iday,mon(i),ihour,':',imins,year0+int(day/365.)
end if
end
 
 
