subroutine ocn(dt_ocn)

!   step of the ocean model

use ocn_vars
use ocn_hbuffer
use ocn_sgs
implicit none

real(4), intent(in) :: dt_ocn
integer icyc, nn
integer it, jt

  dt = dt_ocn
  dtn = dt
  nstep = nstep + 1
  time = time + dt

! refresh reference density profiles for each basin:

  if(mod(nstep, nint(3600./dt)).eq.0) call ocn_refprof()

!------------------------------------------------------------------
!  Check if the dynamical time step should be decreased 
!  to handle the cases when the flow being locally linearly unstable
!------------------------------------------------------------------

  ncycle = 1

  call ocn_kurant()

  do icyc=1,ncycle

     icycle = icyc
     dtn = dt/ncycle
     dt3(na) = dtn
     ncycle3(na) = ncycle
     dtfactor = dtn/dt

     if(mod(nstep,nstatis).eq.0.and.icycle.eq.ncycle) then
        nstatsteps = nstatsteps + 1
        dostatis = .true.
        if(masterproc) print *,'Collecting statistics...'
     else
        dostatis = .false.
     endif

     !bloss:make special statistics flag for radiation,since it's only updated at icycle==1.
     dostatisrad = .false.
     if(mod(nstep,nstatis).eq.0.and.icycle.eq.1) dostatisrad = .true.

!---------------------------------------------
!  	the Adams-Bashforth scheme in time

     call ocn_abcoefs()

!---------------------------------------------
!  	initialize stuff: 

     call ocn_zero()

!------------------------------------------------------------
!       Large-scale and surface forcing:

     call ocn_forcing()

!----------------------------------------------------------
!   	spange-layer damping near the upper boundary:

     call ocn_damping()
!-----------------------------------------------------------
!       Buoyancy term:
     
     if(dodebug) call ocn_debug('before buo')
     call ocn_buoyancy()
     if(dodebug) call ocn_debug('after buo')

!-----------------------------------------------
!       advection of momentum:

     call ocn_advect_mom()
     if(dodebug) call ocn_debug('after advect mom')

!-----------------------------------------------------------
!       Coriolis force:
     
     if(docoriolis) call ocn_coriolis()
     if(dodebug) call ocn_debug('after coriolis')
 
!-----------------------------------------------------------
!  SGS physics:

     if (dosgs) call ocn_sgs_proc()
!----------------------------------------------------------
!     Fill boundaries for SGS diagnostic fields:

     call ocn_boundaries(4)
!----------------------------------------------------------
!	SGS effects on momentum:

     if(dosgs) call ocn_sgs_mom()
     if(dodebug) call ocn_debug('after sgs mom')

!-----------------------------------------------------------
!       surface height gradient effect on momentum:
     
!     call ocn_press_grad_sfc()
!     if(dodebug) call ocn_debug('after sfc press')
!---------------------------------------------------------
!    provisional velocities: apply AB3 and explicit pressure gradient
 
     call ocn_adamsA()

!----------------------------------------------------------
!    Implicit vertical diffusion of momentum:

     call ocn_diffuse_mom_z()

!---------------------------------------------------------
!    iupdate provisional velocities by imcoplete pressure gradient
 
     call ocn_adamsB()

!---------------------------------------------------------
!    apply "projection method" and solve Poisson equation for pressure. 
!    compute non-divergent velocities on n+1 level

     call ocn_pressure()
     if(dodebug) call ocn_debug('after pressure')

!------------------------------------------------------------
!  Update wind and scalar boundaries for scalar advection:

     call ocn_boundaries(1)
     call ocn_boundaries(2)

!---------------------------------------------------------
!      advection of scalars :

     call ocn_advect_all_scalars()

!----------------------------------------------------------
!     Update boundaries for scalars to prepare for SGS effects:

     call ocn_boundaries(3)
   
!---------------------------------------------------------
!      SGS effects on scalars :

     if (dosgs) call ocn_sgs_scalars()

!----------------------------------------------------------
!	Radiation

      if(dolongwave.or.doshortwave) then 
        call ocn_radiation()     
      end if

!-----------------------------------------------------------
!    fill-in scalars inside terrain:

     if(doterrain) call ocn_terrain_fill()

!-----------------------------------------------------------
!    Compute diagnostic fields:

     call ocn_diagnose()

!----------------------------------------------------------

! Rotate the dynamic tendency arrays for Adams-bashforth scheme:

      nn=na
      na=nc
      nc=nb
      nb=nn

   end do ! icycle	

   call ocn_stepout(0)

!----------------------------------------------------------


end 
