subroutine ocn_advect_all_scalars()

use ocn_vars
use ocn_sgs
use ocn_tracers
use ocn_params, only: dotracers
implicit none
real dummy(nz)
real dtdx, dtdy, dtdz, rhox, rhoy, rhoz , a1, a2
integer i,j,k

call t_startf ('ocn_advect_scalars')

!---------------------------------------------------------
!      velocities at n+1/2 time step :

dtdx = dtn/dx
dtdy = dtn/dy
dtdz = dtn/dz
a1 = 0.5
a2 = 0.5
if(nstep.eq.1) then
 a1 = 1.
 a2 = 0.
end if

do k=1,nzm
  rhox = dtdx
  do j=dimy1_u,dimy2_u
   do i=dimx1_u,dimx2_u
     u1(i,j,k) = (a1*u(i,j,k)+a2*u1(i,j,k))*rhox
   end do
  end do
end do
do k=1,nzm
  do j=dimy1_v,dimy2_v
   rhoy = dtdy*muv(j)
   do i=dimx1_v,dimx2_v
     v1(i,j,k) = (a1*v(i,j,k)+a2*v1(i,j,k))*rhoy
   end do
  end do
end do
do k=1,nzm
  do j=dimy1_w,dimy2_w
   rhoz = dtdz*mu(j)
   do i=dimx1_w,dimx2_w
     w1(i,j,k) = (a1*w(i,j,k)+a2*w1(i,j,k))*rhoz
   end do
  end do
end do


!---------------------------------------------------------
!      advection of scalars :

!$OMP PARALLEL DO &
!$OMP DEFAULT(SHARED) &
!$OMP PRIVATE(k,dummy)
     do k = 0,1
       if(k.eq.0) then
        call ocn_advect_scalar(t,tadv,twle,t2leadv,t2legrad,twleadv,.true.)
       else
        call ocn_advect_scalar(s,sadv,swle,s2leadv,s2legrad,swleadv,.true.)
       end if
     end do
!$OMP END PARALLEL DO 
!
!    Advection of sgs prognostics:
!

     if(dosgs.and.advect_sgs) then
       do k = 1,nsgs_fields
           call ocn_advect_scalar(sgs_field(:,:,:,k),sgsadv(:,k),sgswle(:,k),dummy,dummy,dummy,.false.)
       end do
     end if


 ! advection of tracers:

     if(dotracers) then

        do k = 1,ntracers
         call ocn_advect_scalar(tracer(:,:,:,k),tradv(:,k),trwle(:,k),dummy,dummy,dummy,.false.)
        end do

     end if


call t_stopf ('ocn_advect_scalars')

end subroutine ocn_advect_all_scalars
