subroutine ocn_diffuse_scalar_xy (field,fluxb,fluxt,tkh,rho,rhow,flux)

use ocn_params, only: docolumn,dowallx,dowally,dosgs
use ocn_sgs, only: grdf_x,grdf_y,grdf_z, tkmax, tkmin
use ocn_terrain
implicit none
! input	
real field(dimx1_s:dimx2_s, dimy1_s:dimy2_s, nzm)	! scalar
real tkh(0:nxp1,1-YES3D:nyp1,nzm)	! eddy conductivity
real fluxb(nx,ny)		! bottom flux
real fluxt(nx,ny)		! top flux
real rho(nzm)
real rhow(nz)
real flux(nz)
! local        
real flx(0:nx,0:ny,0:nzm)
real dfdt(nx,ny,nz)
real rdx2,rdy2,rdx5,rdy5,rdx5m,rdy5m,tmp
real tkx,tky,tkz,rhoi
integer i,j,k,ib,ic,jb,jc,kc,kb


if(.not.dosgs) return

dfdt(:,:,:)=0.

!-----------------------------------------
if(dowallx) then

  if(mod(rank,nsubdomains_x).eq.0) then
    do k=1,nzm
     do j=1,ny
         field(0,j,k) = field(1,j,k)
     end do
    end do
  end if
  if(mod(rank,nsubdomains_x).eq.nsubdomains_x-1) then
    do k=1,nzm
     do j=1,ny
         field(nx+1,j,k) = field(nx,j,k)
     end do
    end do
  end if

end if

if(dowally) then

  if(rank.lt.nsubdomains_x) then
    do k=1,nzm
       do i=1,nx
         field(i,1-YES3D,k) = field(i,1,k)
       end do
    end do
  end if
  if(rank.gt.nsubdomains-nsubdomains_x-1) then
    do k=1,nzm
       do i=1,nx
         field(i,ny+YES3D,k) = field(i,ny,k)
       end do
    end do
  end if

end if


!-----------------------------------------


!  Horizontal diffusion:

rdx2=1./(dx*dx)
rdy2=1./(dy*dy)

do k=1,nzm

 do j=1,ny
  rdx5=0.5*rdx2
  rdx5m=rdx5*imu(j)**2
  do i=0,nx
    ic=i+1
    tkx=rdx5m*min(2.*tkmax(j,k),max(2.*tkmin(j,k),grdf_x(j,k)*(tkh(i,j,k)+tkh(ic,j,k)))) 	
    flx(i,j,k)=-tkx*(field(ic,j,k)-field(i,j,k))*terrau(ic,j,k)
  end do 
  do i=1,nx
    ib=i-1
    dfdt(i,j,k)=dfdt(i,j,k)-(flx(i,j,k)-flx(ib,j,k))*terra(i,j,k)
  end do 
 end do 

 rdy5=0.5*rdy2
 do j=0,ny
  jc=j+1
  rdy5m = rdy5/adyv(jc)**2*muv(jc)
  do i=1,nx
   tky=rdy5m*min(2.*tkmax(j,k),max(2.*tkmin(j,k),grdf_y(j,k)*(tkh(i,j,k)+tkh(i,jc,k)))) 
   flx(i,j,k)=-tky*(field(i,jc,k)-field(i,j,k))*terrav(i,jc,k)
  end do 
 end do
 do j=1,ny
  jb=j-1
  do i=1,nx	    
    field(i,j,k) = field(i,j,k) + &
      dtn*(dfdt(i,j,k)-imu(j)*(flx(i,j,k)-flx(i,jb,k))*terra(i,j,k))
  end do 
 end do 
 
end do ! k


end subroutine ocn_diffuse_scalar_xy
