module ocn_grid

use domain, only: YES3D, nx_gl, ny_gl, nz_ocn, nsubdomains_x, nsubdomains_y, ocn_ntracers
use ocn_advection, only: NADV, NADVS
use mpi_stuff
use grid, only: case,caseid,caseid_restart,case_restart,ncallocean,  &
    nadams, nrestart,restart_sep,nrestart_skip,&
    day,day0,dx,dy,mu,muv,tanr,imu,imuv,mu_gl,muv_gl,ady,adyv,y_gl,yv_gl, &   
    lon_gl,lat_gl,latv_gl,doyvar

implicit none

character(6), parameter :: version = '7.2.00'
character(8), parameter :: version_date = 'Dec 2017'
        
integer, parameter :: nz_gl = nz_ocn
integer, parameter :: nx = nx_gl/nsubdomains_x
integer, parameter :: ny = ny_gl/nsubdomains_y 
integer, parameter :: nz = nz_gl+1
integer, parameter :: nzm = nz-1

integer, parameter :: nsubdomains = nsubdomains_x * nsubdomains_y

logical, parameter :: RUN3D = ny_gl.gt.1
logical, parameter :: RUN2D = .not.RUN3D

integer, parameter :: nxp1 = nx + 1
integer, parameter :: nyp1 = ny + 1 * YES3D
integer, parameter :: nxp2 = nx + 2
integer, parameter :: nyp2 = ny + 2 * YES3D
integer, parameter :: nxp3 = nx + 3
integer, parameter :: nyp3 = ny + 3 * YES3D
integer, parameter :: nxp4 = nx + 4
integer, parameter :: nyp4 = ny + 4 * YES3D

integer, parameter :: dimx1_u = -1                !!-1        -1        -1        -1
integer, parameter :: dimx2_u = nxp3              !!nxp3      nxp3      nxp3      nxp3
integer, parameter :: dimy1_u = 1-(2+NADV)*YES3D  !!1-5*YES3D 1-4*YES3D 1-3*YES3D 1-2*YES3D
integer, parameter :: dimy2_u = nyp2+NADV         !!nyp5      nyp4      nyp3      nyp2
integer, parameter :: dimx1_v = -1-NADV           !!-4        -3        -2        -1
integer, parameter :: dimx2_v = nxp2+NADV         !!nxp5      nxp4      nxp3      nxp2
integer, parameter :: dimy1_v = 1-2*YES3D         !!1-2*YES3D 1-2*YES3D 1-2*YES3D 1-2*YES3D
integer, parameter :: dimy2_v = nyp3              !!nyp3       nyp3      nyp3      nyp3
integer, parameter :: dimx1_w = -1-NADV           !!-4        -3        -2        -1
integer, parameter :: dimx2_w = nxp2+NADV         !!nxp5      nxp4      nxp3      nxp2
integer, parameter :: dimy1_w = 1-(2+NADV)*YES3D  !!1-5*YES3D 1-4*YES3D 1-3*YES3D 1-2*YES3D
integer, parameter :: dimy2_w = nyp2+NADV         !!nyp5      nyp4      nyp3      nyp2
integer, parameter :: dimx1_s = -2-NADVS          !!-4        -3        -2        -2
integer, parameter :: dimx2_s = nxp3+NADVS        !!nxp5      nxp4      nxp3      nxp3
integer, parameter :: dimy1_s = 1-(3+NADVS)*YES3D !!1-5*YES3D 1-4*YES3D 1-3*YES3D 1-3*YES3D
integer, parameter :: dimy2_s = nyp3+NADVS        !!nyp5      nyp4      nyp3      nyp3

integer, parameter :: ncols = nx*ny

integer, parameter :: ntracers = ocn_ntracers
        
! Vertical grid parameters:
real z(nz)      ! height of the pressure levels above surface,m
real pres(nzm)  ! pressure,mb at scalar levels
real zi(nz)     ! height of the interface levels
real presi(nz)  ! pressure,mb at interface levels
real adz(nzm)   ! ratio of the thickness of scalar levels to dz 
real adzw(nz)	! ratio of the thinckness of w levels to dz
real pres0      ! Reference surface pressure, Pa
real depth_max  ! maximum ocean depth (depth of the first interface level at the bottom)
real depth(nzm) ! depth of scalar levels (m)

real(8):: wgty(1:ny_gl) = 1. ! weight for computing mean column integrals
! weights for global averaging:
real(8):: wgt(ny,nzm) = 1.   
real(8):: wgtu(ny,nzm) = 1.   
real(8):: wgtv(ny,nzm) = 1.  
real(8):: wgtw(ny,nzm) = 1. 

!weights to compute zonal means:
real(8):: wgtx(ny,nzm) = 1.   
real(8):: wgtxu(ny,nzm) = 1.   
real(8):: wgtxv(ny,nzm) = 1.  
real(8):: wgtxw(ny,nzm) = 1.  

integer:: nstep =0! current number of performed time steps 
integer  ncycle  ! number of subcycles over the dynamical timestep
integer icycle  ! current subcycle 
real cfl_adv    ! CFL due to advection
real cfl_advh    ! CFL due to advection (horizontal)
real cfl_advz    ! CFL due to advection (vertical)
real cfl_sgs    ! CFL due to SGS
integer:: na=1, nb=2, nc=3 ! indeces for swapping the rhs arrays for AB scheme
real(8) at, bt, ct ! coefficients for the Adams-Bashforth scheme 
real(8) dtn	! current dynamical timestep (can be smaller than dt)
real(8) dt3(3) 	! dynamical timesteps for three most recent time steps
integer ncycle3(3)      ! number of ncycles for three most recent time steps
real(8):: time=0.	! current time in sec.
real(8) dtfactor   ! dtn/dt
real error_max, error_maxpsy
integer niter_max, niter_maxpsy
logical :: gmg_initialized = .false. 
        
logical dostatis     ! flag to permit the gathering of statistics
logical dostatisrad  ! flag to permit the gathering of radiation statistics
integer nstatis	! the interval between substeps to compute statistics
integer nstatsteps ! counter of statistics stateps

logical notopened2D  ! flag to see if the 2D output datafile is opened	
logical notopened2DZ  ! flag to see if the 2D zonal-mean output datafile is opened	
logical notopened3D  ! flag to see if the 3D output datafile is opened	
logical notopenedmom ! flag to see if the statistical moment file is opened

!-----------------------------------------
! Parameters controled by namelist OCN_PARAMETERS

real:: dz =0.	! constant grid spacing in z direction (when dz_constant=.true.)
logical:: doconstdz = .false.  ! do constant vertical grid spacing set by dz

real(8):: dt=0.	! dynamical timestep

integer:: nstat =1000	! the interval in time steps to compute statistics
integer:: nstatfrq =50 ! frequency of computing statistics 

character(256):: rundatadir ='./RUNDATA' ! path to data directory

integer:: nsave3D =1000     ! frequency of writting 3D fields (steps)
integer:: nsave3Dstart =99999999! timestep to start writting 3D fields
integer:: nsave3Dend  =99999999 ! timestep to end writting 3D fields
logical:: save3Dbin =.false.   ! save 3D data in binary format(no 2-byte compression)
logical:: save3Dsep =.true.   ! use ocn_separate file for each time point for2-model
real   :: qnsave3D =0.    !threshold manimum cloud water(kg/kg) to save 3D fields
logical:: dogzip3D =.false.    ! gzip compress a 3D output file   
logical:: rad3Dout = .false. ! output additional 3D radiation foelds (like reff)

integer:: nsave2D =1000     ! frequency of writting 2D fields (steps)
integer:: nsave2Dstart =99999999! timestep to start writting 2D fields
integer:: nsave2Dend =99999999  ! timestep to end writting 2D fields
logical:: save2Dbin =.false.   ! save 2D data in binary format, rather than compressed
logical:: save2Dsep =.true.   ! write separate file for each time point for 2D output
logical:: save2Davg =.false.   ! flag to time-average 2D output fields (default .false.)
logical:: dogzip2D =.false.    ! gzip compress a 2D output file if save2Dsep=.true.   

integer:: nsave2DZ =1000     ! frequency of writting 2D fields (steps)
integer:: nsave2DZstart =99999999! timestep to start writting 2D fields
integer:: nsave2DZend =99999999  ! timestep to end writting 2D fields
logical:: save2DZbin =.false.   ! save 2D data in binary format, rather than compressed
logical:: save2DZsep =.true.   ! write separate file for each time point for 2D output
logical:: dogzip2DZ =.false.    ! gzip compress a 2D output file if save2Dsep=.true.

!-----------------------------------------
end module ocn_grid
