
subroutine ocn_hyperdiff
	
!        apply hyperdiffusion (4th-derivative) to velocities
!        to control 2-delta waves. In horizontal direction only.
!        Code by Marat Khairoutdinov, 2016

use ocn_vars
use ocn_params, only: dohyper,dohyper_terr, hyper_coef
use ocn_terrain
use ocn_check_energy
implicit none

real rdx4,rdy4

integer i,j,k,ja,jb,jd,je,it,jt
real a(ny),b(ny),c(ny),d(ny),e(ny)
real rd,rdt,ddd,rc0,rc1,rc0t

call ocn_sumenergy(1,'ocn_hyper')
!-----------------------------------------

if(dohyper) then
  rc0=1./hyper_coef ! dimensionless dhyperdiff coef outside the poles
  rc1=1./hyper_coef ! dimensionless dhyperdiff coef around the poles
else
  rc0=0.
  rc1=0.
end if

if(dohyper_terr) then
  rc0t=1./128. ! dimensionless dhyperdiff close to terrain
else
  rc0t=rc1
end if

rc0=rc0/dtn
rc1=rc1/dtn
rc0t=rc0t/dtn

do k=1,nzm
  do j=1,ny
   rd=max(rc0,rc1*(1.-mu(j)**2)**50)
   do i=1,nx
    ddd = min(terrau(i-2,j,k),terrau(i-1,j,k),terrau(i,j,k),terrau(i+1,j,k),terrau(i+2,j,k))
    if(ddd.ne.0) then
       dudt(i,j,k,na)=dudt(i,j,k,na)- &
             rd*(u(i-2,j,k)-4.*u(i-1,j,k)+6.*u(i,j,k)-4.*u(i+1,j,k)+u(i+2,j,k))
    else
       dudt(i,j,k,na)=dudt(i,j,k,na)- &
             rc0t*(u(i-2,j,k)-4.*u(i-1,j,k)+6.*u(i,j,k)-4.*u(i+1,j,k)+u(i+2,j,k))
    end if
   end do
  end do
end do

do k=1,nzm
  do j=1,ny
   rd=max(rc0,rc1*(1.-muv(j)**2)**50)
   do i=1,nx
    ddd = min(terrav(i-2,j,k),terrav(i-1,j,k),terrav(i,j,k),terrav(i+1,j,k),terrav(i+2,j,k))
    if(ddd.ne.0.) then
      dvdt(i,j,k,na)=dvdt(i,j,k,na)- &
         rd*(v(i-2,j,k)-4.*v(i-1,j,k)+6.*v(i,j,k)-4.*v(i+1,j,k)+v(i+2,j,k))
    else
      dvdt(i,j,k,na)=dvdt(i,j,k,na)- &
         rc0t*(v(i-2,j,k)-4.*v(i-1,j,k)+6.*v(i,j,k)-4.*v(i+1,j,k)+v(i+2,j,k))
    end if
   end do
  end do
end do

do k=2,nzm
  do j=1,ny
   rd=max(rc0,rc1*(1.-mu(j)**2)**50)
   do i=1,nx
    ddd = min(terraw(i-2,j,k),terraw(i-1,j,k),terraw(i,j,k),terraw(i+1,j,k),terraw(i+2,j,k))
    if(ddd.ne.0) then
       dwdt(i,j,k,na)=dwdt(i,j,k,na)- &
             rd*(w(i-2,j,k)-4.*w(i-1,j,k)+6.*w(i,j,k)-4.*w(i+1,j,k)+w(i+2,j,k))
    else
       dwdt(i,j,k,na)=dwdt(i,j,k,na)- &
             rc0t*(w(i-2,j,k)-4.*w(i-1,j,k)+6.*w(i,j,k)-4.*w(i+1,j,k)+w(i+2,j,k))
    end if
   end do
  end do
end do

call task_rank_to_index(rank,it,jt)

if(RUN2D) return


!!!!

return

!!!



do j=1,ny
 if(jt+j.gt.2.and.jt+j.lt.ny_gl-1) then
  a(j) = imu(j)*imu(j-1)*muv(j)*muv(j-1)/ady(j)/ady(j-1)*adyv(j)*adyv(j-1)
  b(j) = imu(j)*muv(j)/ady(j)*adyv(j)*(imu(j)/ady(j)*(muv(j)*adyv(j)+muv(j+1)*adyv(j+1)) &
             +imu(j-1)/ady(j-1)*(muv(j)*adyv(j)+muv(j-1)*adyv(j-1)))
  c(j) = imu(j)/ady(j)*(imu(j-1)*muv(j)*muv(j)/ady(j-1)*adyv(j)*adyv(j) &
             +imu(j)/ady(j)*(muv(j)*adyv(j)+muv(j+1)*adyv(j+1))**2 &
             +imu(j+1)*muv(j+1)*muv(j+1)/ady(j+1)*adyv(j+1)*adyv(j+1))
  d(j) = imu(j)*muv(j+1)/ady(j)*adyv(j+1)*(imu(j)/ady(j)*(muv(j)*adyv(j)+muv(j+1)*adyv(j+1)) &
             +imu(j+1)/ady(j+1)*(muv(j+1)*adyv(j+1)+muv(j+2)*adyv(j+2)))
  e(j) = imu(j)*imu(j+1)*muv(j+1)*muv(j+2)/ady(j)/ady(j+1)*adyv(j+1)*adyv(j+2)
 else
  a(j) = 0.
  b(j) = 0.
  c(j) = 0.
  d(j) = 0.
  e(j) = 0.
 end if
end do

do k=1,nzm
  do j=1,ny
   ja=j-2*YES3D
   jb=j-YES3D
   jd=j+YES3D
   je=j+2*YES3D
   rd=rc0
!   rd=max(rc0,rc1*(1.-mu(j)**2)**50)
   do i=1,nx
    ddd = min(terrau(i,ja,k),terrau(i,jb,k),terrau(i,j,k),terrau(i,jd,k),terrau(i,je,k))
    if(ddd.ne.0.) then
      dudt(i,j,k,na)=dudt(i,j,k,na)- &
             rd*(a(j)*u(i,ja,k)-b(j)*u(i,jb,k)+c(j)*u(i,j,k)-d(j)*u(i,jd,k)+e(j)*u(i,je,k))
    else
      dudt(i,j,k,na)=dudt(i,j,k,na)- &
             rc0t*(a(j)*u(i,ja,k)-b(j)*u(i,jb,k)+c(j)*u(i,j,k)-d(j)*u(i,jd,k)+e(j)*u(i,je,k))
    end if
   end do
  end do
end do

do k=2,nzm
  do j=1,ny
   ja=j-2*YES3D
   jb=j-YES3D
   jd=j+YES3D
   je=j+2*YES3D
   rd=rc0
!   rd=max(rc0,rc1*(1.-mu(j)**2)**50)
   do i=1,nx
    ddd = min(terraw(i,ja,k),terraw(i,jb,k),terraw(i,j,k),terraw(i,jd,k),terraw(i,je,k))
    if(ddd.ne.0.) then
      dwdt(i,j,k,na)=dwdt(i,j,k,na)- &
             rd*(a(j)*w(i,ja,k)-b(j)*w(i,jb,k)+c(j)*w(i,j,k)-d(j)*w(i,jd,k)+e(j)*w(i,je,k))
    else
      dwdt(i,j,k,na)=dwdt(i,j,k,na)- &
             rc0t*(a(j)*w(i,ja,k)-b(j)*w(i,jb,k)+c(j)*w(i,j,k)-d(j)*w(i,jd,k)+e(j)*w(i,je,k))
    end if
   end do
  end do
end do



do j=1,ny
 if(jt+j.gt.2.and.jt+j.lt.ny_gl) then
  a(j) = imuv(j)*imuv(j-1)*mu(j-1)*mu(j-2)/adyv(j)/adyv(j-1)*ady(j-1)*ady(j-2)
  b(j) = imuv(j)*mu(j-1)/adyv(j)*ady(j-1)*(imuv(j)/adyv(j)*(mu(j-1)*ady(j-1)+mu(j)*ady(j)) &
         +imuv(j-1)/adyv(j-1)*(mu(j-1)*ady(j-1)+mu(j-2)*ady(j-2)))
  c(j) = imuv(j)/adyv(j)*(imuv(j-1)*mu(j-1)*mu(j-1)/adyv(j-1)*ady(j-1)*ady(j-1) &
         +imuv(j)/adyv(j)*(mu(j-1)*ady(j-1)+mu(j)*ady(j))**2 &
         +imuv(j+1)*mu(j)*mu(j)/adyv(j+1)*ady(j)*ady(j))
  d(j) = imuv(j)*mu(j)/adyv(j)*ady(j)*(imuv(j)/adyv(j)*(mu(j-1)*ady(j-1)+mu(j)*ady(j)) &
         +imuv(j+1)/adyv(j+1)*(mu(j)*ady(j)+mu(j+1)*ady(j+1)))
  e(j) = imuv(j)*imuv(j+1)*mu(j)*mu(j+1)/adyv(j)/adyv(j+1)*ady(j)*ady(j+1)
 else
  a(j) = 0.
  b(j) = 0.
  c(j) = 0.
  d(j) = 0.
  e(j) = 0.
 end if
end do


do k=1,nzm
  do j=1,ny
   ja=j-2*YES3D
   jb=j-YES3D
   jd=j+YES3D
   je=j+2*YES3D
   rd=rc0
!   rd=max(rc0,rc1*(1.-muv(j)**2)**50)
   do i=1,nx
    ddd = min(terrav(i,ja,k),terrav(i,jb,k),terrav(i,j,k),terrav(i,jd,k),terrav(i,je,k))
    if(ddd.ne.0.) then
       dvdt(i,j,k,na)=dvdt(i,j,k,na)- &
             rd*(a(j)*v(i,ja,k)-b(j)*v(i,jb,k)+c(j)*v(i,j,k)-d(j)*v(i,jd,k)+e(j)*v(i,je,k))
    else
       dvdt(i,j,k,na)=dvdt(i,j,k,na)- &
             rc0t*(a(j)*v(i,ja,k)-b(j)*v(i,jb,k)+c(j)*v(i,j,k)-d(j)*v(i,jd,k)+e(j)*v(i,je,k))
    end if
   end do
  end do
end do

call ocn_sumenergy(-1,'ocn_hyper')

end subroutine ocn_hyperdiff
