module ocn_params

use params, only: earth_factor, doglobal, dolatlon, docoriolis, &
                  dowallx, dowally, doperpetual, doflat, doregion
use consts
implicit none

!
! internally set parameters:

integer:: ncycle_max = 50  ! maximum number of subcycling within dt

!----------------------------------------------
! Parameters set by PARAMETERS namelist:
! Initialized to default values.
!----------------------------------------------
logical:: dodebug = .false.

logical:: doimplicitdiff=.true. ! switch to implicit diffusion in z

real(8):: tabs_s =0.    ! surface temperature,K

real::   fluxt0 =0.  ! surface sensible flux, W/m2
real::   fluxq0 =0.  ! surface latent flux, W/m2
real::   taux0   =0.  ! surface zonal stress, m2/s2
real::   tauy0   =0.  ! surface meridional stress, m2/s2
real::   z0     = 0.035 ! default roughness length (m)
logical:: readinit = .false.   ! read 3D initial conditions from file
logical:: readq = .false. ! read air q from file
logical:: readt = .false. ! read air t from file
logical:: readroff = .false. ! read river runoff from file
logical:: readlhf = .false. ! read lhf from file
logical:: readshf = .false. ! read shf from file
logical:: readprec = .false. ! read precip from file
logical:: readlw = .false. ! read net lw from file
logical:: readsw = .false. ! read net sw from file
logical:: readu = .false. ! read air u from file
logical:: readv = .false. ! read air v from file
logical:: readtaux = .false. ! read taux from file
logical:: readtauy = .false. ! read tauy from file
logical:: readterr = .false.   ! read terrain
logical:: readbasins = .false.   ! read basins
character(80):: initfile = ""
character(80):: terrainfile = ""
character(80):: qfile = ""
character(80):: tfile = ""
character(80):: rofffile = ""
character(80):: lhffile = ""
character(80):: shffile = ""
character(80):: precfile = ""
character(80):: lwfile = ""
character(80):: swfile = ""
character(80):: ufile = ""
character(80):: vfile = ""
character(80):: tauxfile = ""
character(80):: tauyfile = ""
character(80):: basinsfile = ""

logical:: sfc_flx_fxd =.false. ! surface sensible flux is fixed
logical:: sfc_tau_fxd =.false.! surface drag is fixed

real:: timelargescale =0. ! time to start large-scale forcing

! nudging boundaries (between z1 and z2, where z2 > z1): 
real:: nudging_uv_z1 =-1., nudging_uv_z2 = 1000000.
real:: nudging_t_z1 =-1., nudging_t_z2 = 1000000.
real:: nudging_s_z1 =-1., nudging_s_z2 = 1000000.
real:: tauls = 99999999.    ! nudging-to-large-scaler-profile time-scale
real:: tautsls = 99999999.! nudging-to-large-scaler-profile time-scale for scalars

real:: bufferzonex = 0. ! fraction of the domain in x for nudging zone
real:: bufferzoney = 0. ! fraction of the domain in y for nudging zone

real:: damping_u_cu = 0.40 ! Courant number used in damping of horizontal velocity (dodamping_pole=.true.)
real:: damping_w_cu = 0.35 ! Courant number used in damping of vertical velocity (dodamping_w=.true.)
real:: tk_factor = 0. ! factor to compute minimum tk for momentum (in tke_full.f90)
real:: hyper_coef = 512. ! dimensional coefficient for hyper diffusion (1./hyper_coef)
real:: gmg_precision = 1.e-6 ! precision of GMG solver

logical:: dodamping = .false.
logical:: dodamping_poles = .false.
logical:: dodamping_w = .false.
logical:: dodamping_u = .false.
logical:: dohyper = .false. ! apply 4th-order hyperviscosity to filter 2-delta waves
logical:: dohyper_terr = .false. ! apply 4th-order hyperviscosity near terrain
logical:: doupperbound = .false. 
logical:: dolongwave = .false. 
logical:: doshortwave = .false.
logical:: dosgs = .false.
logical:: dosurface = .false. 
logical:: dolargescale = .false. 
logical:: doradforcing = .false.
logical:: dosfcforcing = .false. 
logical:: doradsimple = .false. 
logical:: donudging_uv = .false. 
logical:: donudging_ts = .false.
logical:: donudging_t = .false. 
logical:: donudging_s = .false.
logical:: doensemble = .false. 
logical:: docolumn = .false. 
logical:: docup = .false.
logical:: doradhomo = .false. 
logical:: dosfchomo = .false.
logical:: dossthomo = .false. 
logical:: dotracers = .false. 
logical:: doterrain = .false.
logical:: dobufferzone = .false.
logical:: dopointsource = .false.
integer:: pointsource_start_step = 1

integer:: nensemble =0   ! the number of subensemble set of perturbations
integer:: perturb_type  = 0 ! type of initial noise in setperturb()
integer:: nens = 0          ! add this number to seed in setperturb to make ensembles
integer:: terrain_type = 0 ! type of the terrain initialization

! Initial bubble parameters. Activated when perturb_type = 2
  real:: bubble_x0 = 0.
  real:: bubble_y0 = 0.
  real:: bubble_z0 = 0.
  real:: bubble_radius_hor = 0.
  real:: bubble_radius_ver = 0.
  real:: bubble_dtemp = 0.
  real:: bubble_ds = 0.


end module ocn_params
