
subroutine ocn_radiation
 	
!	Simple Radiation Transfer in ocean layer
! Absorption is modeled after Lee et al (2005) JGR
! "Penetration of solar radiation in the upper ocean: A numerical model for oceanic and coastal waters"
! Coded by Marat Khairoutdinov, Oct 2013


use ocn_vars
use ocn_params
use vars, only: sw_ocn, lw_ocn, landmask
use grid, only : day
use ocn_terrain, only: k_terra
use slm_vars, only: seaicemask
implicit none
	
! for various value of Chlorophyll concentration, from low to high
!real,parameter :: aaa=0.029, bbb=0.0057
real,parameter :: aaa=0.057, bbb=0.0068
!real,parameter :: aaa=0.137, bbb=0.0081
real fr(nx,ny),xk, xk1, xk2, fr1, dtrad, coef, cosz, pii
integer i,j,k
	

pii = acos(-1.)

radswdn(:) =0.
radqrsw(:) =0.
radlwdn(:) =0.
radqrlw(:) =0.
qrad(:,:,:) = 0.

if(doshortwave) then
	
radswdn(nz) = radswdn(nz)+sum(sw_ocn(1:nx,1:ny))
swns_xy(:,:) = swns_xy(:,:) + sw_ocn(:,:)*dtfactor

! Solar (visible):
fr(:,:) = 0.5*sw_ocn(:,:) ! assume that visible is  about 50% of solar 
do k = nzm,1,-1
  coef = 1./(cpw*1000.*dz*adz(k))
  do j=1,ny
  do i=1,nx
   if(landmask(i,j).eq.0..and.seaicemask(i,j).eq.0.and.k.ge.k_terra(i,j)) then
    if(doperpetual) then
      cosz=1.
    else
      cosz=cosza(real(day),real(pii/180.*latitude(i,j)),real(pii/180.*longitude(i,j)))
    end if
    if(cosz.gt.0.) then
     xk1 = (-0.057+0.482*sqrt(aaa)+4.221*bbb)*(1.+0.09*sqrt(1.-cosz*cosz))
     xk2 = (0.183+0.702*aaa-2.567*bbb)*(1.465-0.667*cosz)
     xk = xk1 + xk2/sqrt(1.+zi(nz)-z(k))
     fr1 = fr(i,j)*exp(xk*(zi(k)-zi(k+1)))
     dtrad = (fr(i,j) - fr1) * coef
     fr(i,j) = fr1
     t(i,j,k) = t(i,j,k) + dtrad * dtn
     qrad(i,j,k) = qrad(i,j,k)+dtrad
     radswdn(k) = radswdn(k) + fr1
     radqrsw(k) = radqrsw(k) + dtrad
     if(fr1.lt.0.001) exit
    end if
   end if
  end do
  end do
end do

! Near IR: (not thermal radiation!)

fr(:,:) = 0.5*sw_ocn(:,:)
do k = nzm,1,-1
  coef = 1./(cpw*1000.*dz*adz(k))
  xk = 0.56+2.304/(0.001+zi(nz)-z(k))**0.65
  do j=1,ny
  do i=1,nx
   if(landmask(i,j).eq.0..and.seaicemask(i,j).eq.0.and.k.ge.k_terra(i,j)) then
    fr1 = fr(i,j) * exp(xk*(zi(k)-zi(k+1)))
    dtrad = (fr(i,j) - fr1) * coef
    fr(i,j) = fr1
    t(i,j,k) = t(i,j,k) + dtrad * dtn
    qrad(i,j,k) = qrad(i,j,k) + dtrad
    radswdn(k) = radswdn(k) + fr1
    radqrsw(k) = radqrsw(k) + dtrad
    if(fr1.lt.0.001) exit
   end if
  end do
  end do
end do

end if


if(dolongwave) then

lwns_xy(:,:) = lwns_xy(:,:) + lw_ocn(:,:)*dtfactor
fr(:,:) = lw_ocn(:,:)
radlwdn(nz) = radlwdn(nz) + sum(lw_ocn(1:nx,1:ny))
coef = 1./(cpw*1000.*dz*adz(nzm))
do j=1,ny
  do i=1,nx
   if(landmask(i,j).eq.0.and.seaicemask(i,j).eq.0) then
    dtrad = fr(i,j)*coef
    t(i,j,nzm) = t(i,j,nzm) + dtrad * dtn 
    qrad(i,j,nzm) = qrad(i,j,nzm) + dtrad
    radqrlw(nzm) = radqrlw(nzm) + dtrad
   end if
  end do
end do


end if

	

contains 
real function cosza(calday, clat, clon)
     use shr_kind_mod, only: r4 => shr_kind_r4
     use shr_orb_mod, only : shr_orb_decl, shr_orb_cosz
     implicit none
     real, intent(in ) :: calday, & ! Calendar day, including fraction
                                   clat,   & ! Current centered latitude (radians)
                                   clon      ! Centered longitude (radians)

     real(r4) ::  eccf,  & ! eccentricity factor (1./earth-sun dist^2)
                 eccen, & ! Earth's eccentricity factor (unitless) (typically 0 to 0.1)
                 obliq, & ! Earth's obliquity angle (deg) (-90 to +90) (typically 22-26)
                 mvelp, & ! Earth's moving vernal equinox at perhelion (deg)(0 to 360.0)
                 !
                 ! Orbital information after processed by orbit_params
                 !
                 obliqr, &  ! Earth's obliquity in radians
                 lambm0, &  ! Mean longitude of perihelion at the
                            ! vernal equinox (radians)
                 mvelpp     ! Earth's moving vernal equinox longitude
                            ! of perihelion plus pi (radians)
     real(r4):: delta ! Solar declination angle in radians

     call shr_orb_decl (real(calday,r4), eccen, mvelpp, lambm0, obliqr, delta, eccf)
     !
     ! Compute local cosine solar zenith angle
     !
     cosza = shr_orb_cosz(real(calday,r4), real(clat,r4), real(clon,r4), delta)
end function

end subroutine 
