subroutine ocn_setgrid

! Initialize vertical grid

use ocn_vars	
use ocn_params
use ocn_terrain, only: ocn_setterrain, terra, terrau, terrav, terraw

implicit none
	
real latit, long, phi, zz
integer i,j,it,jt,k, kk
real(8) sums(nzm),sumu(nzm),sumv(nzm),sumw(nzm)
real(8) coef, coef1, buffer(nzm,4), buffer1(nzm,4)
real tmpz(ny,nzm)

if(nrestart.eq.0) then

 
 if(doconstdz) then

    do k=2,nz
     zi(k)=dz*(k-1)
    end do
    do k=1,nzm
      adz(k) = (zi(k+1)-zi(k))/dz
      z(k) = 0.5*(zi(k+1)+zi(k))
    end do
    do k=2,nzm
      adzw(k) = (z(k)-z(k-1))/dz
    end do
    adzw(1) = 1.
    adzw(nz) = adzw(nzm)

 else

    open(8,file='./CASES/'//trim(case)//'/grd_ocn',status='old',form='formatted') 
    read(8,*) zi(nz)
    rewind(8)

    if(zi(nz).ne.0.) then
      if(masterproc) print*,'grid in grd_ocn sets midlevels'
      zi(1) = 0.
      do k=1,nzm
        read(8,fmt=*,end=222) z(k)
        kk=k
      end do
222   do k=kk,nzm-1 
        z(k+1) = z(k)+(z(kk)-z(kk-1))
      end do
      do k=2,nzm
        zi(k) = 0.5*(z(k-1)+z(k))
      end do
      zi(nz) = zi(nzm)+(zi(nzm)-zi(nzm-1))
    else
      if(masterproc) print*,'grid in grd_ocn sets interface levels'
      do k=1,nz
        read(8,fmt=*,end=333) zi(k)
        kk=k
      end do
333   do k=kk,nz-1
       zi(k+1)=zi(k)+(zi(kk)-zi(kk-1))
      end do
444   continue
    end if
    close (8)
    do k=1,nzm
     z(k)=0.5*(zi(k)+zi(k+1))
    end do
    zz = zi(nz)
    z(1:nzm) = zz-z(nzm:1:-1) 
    zi(1:nz) = zz-zi(nz:1:-1) 
    dz = zi(nz)-zi(nzm)
    do k=1,nzm
      adz(k) = (zi(k+1)-zi(k))/dz
      z(k) = 0.5*(zi(k+1)+zi(k))
      depth(k) = zi(nz)-z(k)
    end do
    do k=2,nzm
      adzw(k) = (z(k)-z(k-1))/dz
    end do
    adzw(nz) = adzw(nzm)
    adzw(1) = 1
    if(masterproc) then
      print*,'adz:',adz(1:nzm)
      print*,'adzw:',adzw(1:nz)
      print*,'z:',z(1:nzm)
      print*,'zi:',zi(1:nz)
      print*,'depth:',zi(nz)-z(1:nzm)
    end if
 end if

end if

pres(1:nzm)=ggr*(zi(nz)-z(1:nzm))*0.1   ! pressure in decibars
presi(1:nz)=ggr*(zi(nz)-zi(1:nz))*0.1   ! pressure in decibars
rho(:) = 1000.
rhow(:) = 1000.
bet(:) = ggr/1000.
do k=1,nzm
 p3D(:,:,k) = pres(k)
end do
if(masterproc) then
 print*,'pres=',pres
 print*,'presi=',presi
end if
! Initialize terrain

call ocn_setterrain()


if(z(nzm).gt.40000.) then
  if(masterproc) print*,'the domain top is too high: z(nzm) = ',z(nzm), '   exit...'
  call task_abort()
end if

!--------------------------------------------
! compute weights for global averaging:
sums = 0.
sumu = 0.
sumv = 0.
sumw = 0.
do k=1,nzm
 do j=1,ny
  do i=1,nx
    sums(k) = sums(k) + mu(j)*ady(j)*terra(i,j,k)
    sumu(k) = sumu(k) + mu(j)*ady(j)*terrau(i,j,k)
    sumv(k) = sumv(k) + muv(j)*adyv(j)*terrav(i,j,k)
    sumw(k) = sumw(k) + mu(j)*ady(j)*terraw(i,j,k)
  end do
 end do
end do
if(dompi) then
  buffer(:,1) = sums(:)
  buffer(:,2) = sumu(:)
  buffer(:,3) = sumv(:)
  buffer(:,4) = sumw(:)
  call task_sum_real8(buffer,buffer1,nzm*4)
  sums(:) = buffer1(:,1)
  sumu(:) = buffer1(:,2)
  sumv(:) = buffer1(:,3)
  sumw(:) = buffer1(:,4)
end if
if(masterproc) then
  write(*,*) 'k,sumu(k),sumv(k),sumw(k),sums(k)'
  do k=1,nzm
   write(*,'(i3,4g15.7)') k,sumu(k),sumv(k),sumw(k),sums(k)
  end do
end if
if(masterproc.and.(minval(sums).eq.0..or.minval(sumu).eq.0. &
                .or.minval(sumv).eq.0..or.minval(sumw).eq.0.)) then
    print*,'error in computing wgts in setgrid: terrain cannot be everywhere!'
    call task_abort()
end if
do k=1,nzm
 do j=1,ny
   wgt(j,k) = mu(j)*ady(j)*dble(nx_gl*ny_gl)/sums(k)
   wgtu(j,k) = mu(j)*ady(j)*dble(nx_gl*ny_gl)/sumu(k)
   wgtv(j,k) = muv(j)*adyv(j)*dble(nx_gl*ny_gl)/sumv(k)
   wgtw(j,k) = mu(j)*ady(j)*dble(nx_gl*ny_gl)/sumw(k)
 end do
end do

!--------------------------------------------
! compute weights for global averaging:

call ocn_mean_x_3D(terrau(1:nx,1:ny,1:nzm),wgtxu)
wgtxu(:,:)=1./(wgtxu(:,:)+1.e-10)

call ocn_mean_x_3D(terrav(1:nx,1:ny,1:nzm),wgtxv)
wgtxv(:,:)=1./(wgtxv(:,:)+1.e-10)

call ocn_mean_x_3D(terraw(1:nx,1:ny,1:nzm),wgtxw)
wgtxw(:,:)=1./(wgtxw(:,:)+1.e-10)

call ocn_mean_x_3D(terra(1:nx,1:ny,1:nzm),wgtx)
wgtx(:,:)=1./(wgtx(:,:)+1.e-10)

if(masterproc) then
  print*,'wgtxu',minval(wgtxu),maxval(wgtxu)
  print*,'wgtxv',minval(wgtxv),maxval(wgtxv)
  print*,'wgtxw',minval(wgtxw),maxval(wgtxw)
  print*,'wgtx',minval(wgtx),maxval(wgtx)
end if

end
