! compute surface fluxes when ocean model only.

subroutine ocn_surface()

use ocn_vars
use ocn_params
use simple_ocean, only: sst_evolve
use slm_vars, only: seaicemask
use vars, only: sstxy, landmask, lhf_ocn, shf_ocn, taux_ocn, tauy_ocn
use params, only: salt_factor
use ocn_seaice
implicit none

real hh, t_s, q_s, w_s, r_s, z0_s, t_h, q_h, ta_h, u_h, v_h, u_h0, rho_h
real taux, tauy, xlmo
real diag_ustar, trf,qrf,urf,vrf,ra,rs
integer i,j,k,iqv
real evapi, tmpu(0:nx,ny), tmpv(nx,0:ny) 

integer tag(2),req(2),irank,count
real(8) buffer(2), buffer1(2)
real buffers(max(nx,ny),2)
logical flag(2)
real, external :: ocn_qsatw, ocn_qsati

call t_startf ('ocn_surface')

hh = 10.1 ! height of atmospheric reference level

do j=1,ny
   do i=1,nx

     if(landmask(i,j).eq.0) then

           call ocn_seaice_init(h_seaice(i,j), tabs(i,j,nzm), zi(nz)-zi(nzm),  &
                                s(i,j,nzm), rhoo(i,j,nzm), t_seaice(i,j))
           if(h_seaice(i,j).gt.0.) then
              seaicemask(i,j) = 1
           else
              seaicemask(i,j) = 0
           end if

           u_h = u_atm(i,j)-u(i,j,nzm)
           v_h = v_atm(i,j)-v(i,j,nzm)
           ta_h = t_atm(i,j)
           t_h = t_atm(i,j)+9.8e-3*hh
           rho_h = 101325./(287.*ta_h)
           q_h = q_atm(i,j)
           if(seaicemask(i,j).eq.0) then
             t_s = sstxy(i,j)+273.16
             q_s = salt_factor*ocn_qsatw(t_s,1013.)
           else
             t_s = t_h+(tabs(i,j,nzm)+273.15-t_h)*exp(-h_seaice(i,j))
             q_s = ocn_qsati(t_s,1013.)
           end if
           call ocn_oceflx(rho_h, u_h, v_h, ta_h, q_h, t_h, hh, t_s, q_s, &
                        fluxt0, fluxq0, taux, tauy, ra, trf, qrf, urf, vrf)

           if(seaicemask(i,j).eq.0) then
             shf_ocn(i,j) = fluxt0*cp*rho_h
             lhf_ocn(i,j) = fluxq0*lcond*rho_h
             taux_ocn(i,j) = taux
             tauy_ocn(i,j) = tauy
           else
             fluxt0 = (fluxt0*cp + fluxq0*lcond)*rho_h/(cpw*rhow(nz))
             tabs(i,j,nzm) = tabs(i,j,nzm) - dtn*fluxt0/(dz*adz(nzm)) ! cool water first
             call ocn_seaice_proc(h_seaice(i,j), tabs(i,j,nzm), zi(nz)-zi(nzm),  &
                                  s(i,j,nzm), rhoo(i,j,nzm), t_seaice(i,j))
             evapi = dtn * fluxq0 * rho_h ! ice evaporation over time step (kg)
             h_seaice(i,j) = max(0., h_seaice(i,j) - 0.001*evapi*1.075)
             if(h_seaice(i,j).eq.0.) seaicemask(i,j) = 0.
             shf_ocn(i,j) = 0.
             lhf_ocn(i,j) = 0.
             taux_ocn(i,j) = 0.
             tauy_ocn(i,j) = 0.
           end if

     end if

   end do
end do
! 
! compute stresses at velocity positions:

  tmpu(1:nx,1:ny) = taux_ocn(1:nx,1:ny)
  tmpv(1:nx,1:ny) = tauy_ocn(1:nx,1:ny)
  if(dompi) then
     call ocn_task_receive_float(buffers(:,1),max(nx,ny),req(1))
     call ocn_task_receive_float(buffers(:,2),max(nx,ny),req(2))
     call ocn_task_bsend_float(rankee,real(taux_ocn(nx,1:ny)),ny,133)
     call ocn_task_bsend_float(ranknn,real(tauy_ocn(1:nx,ny)),nx,134)
     count = 2  
     flag(1:2) = .false.
     do while(count.gt.0)
      do k=1,2
        if(.not.flag(k)) then
          call task_test(req(k),flag(k),irank,tag(k))
          if(flag(k)) then
            if(tag(k).eq.133) then
             tmpu(0,1:ny) = buffers(1:ny,k)  
             count=count-1
            else if(tag(k).eq.134) then
             tmpv(1:nx,0) = buffers(1:nx,k)  
             count=count-1
            else
              print*,'surface: wrong tag. Should be 133 or 134, got ',tag
              call task_abort
            end if
          end if
        end if 
      end do
     end do
  else
     tmpu(0,1:ny) = taux_ocn(nx,1:ny)
     tmpv(1:nx,0) = tauy_ocn(1:nx,ny)
  end if
  do j=1,ny
   do i=1,nx
    taux_ocn(i,j) = 0.5*(tmpu(i,j)+tmpu(i-1,j))
   end do
  end do

  do j=1,ny
   do i=1,nx
    tauy_ocn(i,j) = 0.5*(tmpv(i,j)+tmpv(i,j-YES3D)) 
   end do
  end do

  call task_barrier()

call t_stopf ('ocn_surface')

end

real function ocn_esatw(t)
implicit none
real t  ! temperature (K)
real a0,a1,a2,a3,a4,a5,a6,a7,a8
data a0,a1,a2,a3,a4,a5,a6,a7,a8 /&
!       6.105851, 0.4440316, 0.1430341e-1, &
!        0.2641412e-3, 0.2995057e-5, 0.2031998e-7, &
!        0.6936113e-10, 0.2564861e-13,-0.3704404e-15/
        6.11239921, 0.443987641, 0.142986287e-1, &
       0.264847430e-3, 0.302950461e-5, 0.206739458e-7, &
       0.640689451e-10, -0.952447341e-13,-0.976195544e-15/
real dt
dt = max(-80.,t-273.16)
ocn_esatw = a0 + dt*(a1+dt*(a2+dt*(a3+dt*(a4+dt*(a5+dt*(a6+dt*(a7+a8*dt)))))))
end



real function ocn_qsatw(t,p)
implicit none
real t  ! temperature (K)
real p  ! pressure    (mb)
real esat,ocn_esatw
esat = ocn_esatw(t)
ocn_qsatw = 0.622 * esat/max(esat,p-esat)
end

real function ocn_esati(t)
implicit none
real t  ! temperature (K)
real a0,a1,a2,a3,a4,a5,a6,a7,a8
data a0,a1,a2,a3,a4,a5,a6,a7,a8 /&
        6.11147274, 0.503160820, 0.188439774e-1, &
        0.420895665e-3, 0.615021634e-5,0.602588177e-7, &
        0.385852041e-9, 0.146898966e-11, 0.252751365e-14/
real dt
real ocn_esatw
if(t.gt.273.15) then
  ocn_esati = ocn_esatw(t)
else if(t.gt.185.) then
  dt = t-273.16
  ocn_esati = a0 + dt*(a1+dt*(a2+dt*(a3+dt*(a4+dt*(a5+dt*(a6+dt*(a7+a8*dt)))))))
else   ! use some additional interpolation below 184K
  dt = max(-100.,t-273.16)
  ocn_esati = 0.00763685 + dt*(0.000151069+dt*7.48215e-07)
end if
end



real function ocn_qsati(t,p)
implicit none
real t  ! temperature (K)
real p  ! pressure    (mb)
real esat,ocn_esati
esat=ocn_esati(t)
ocn_qsati=0.622 * esat/max(esat,p-esat)
end

