module ocn_terrain

!incorporation of terrain into SAM
!Author: Marat Khairoutdinov, Nov. 2011
!
! note, that unlike terrain for land, for ocean terrain elevation should be negative
! relative to zero at the ocean surface.
!--------------------------------------------------------------------

use ocn_grid
use ocn_vars, only: elevation, elevationg
use ocn_params, only: terrainfile, terrain_type, doterrain, pi,dowallx,dowally
use params, only: dooceanonly,readlandmask
use vars, only: landmask

implicit none

! terrain masks: inside terrain? (1 no, 0 yes, in between - kind of)
real terrau   (dimx1_u:dimx2_u, dimy1_u:dimy2_u, nzm) 
real terrav   (dimx1_v:dimx2_v, dimy1_v:dimy2_v, nzm) 
real terraw   (dimx1_w:dimx2_w, dimy1_w:dimy2_w, nz ) 
real terra    (dimx1_s:dimx2_s, dimy1_s:dimy2_s, nzm) 

integer terr_index(nx, ny) ! vertical index
integer k_terra(dimx1_s:dimx2_s, dimy1_s:dimy2_s) ! vertical grid index of terrain top
integer k_terrau(dimx1_u:dimx2_u, dimy1_u:dimy2_u) 
integer k_terrav(dimx1_v:dimx2_v, dimy1_v:dimy2_v) 

integer kmax ! maximum vertical index of terrain in the current subdomain
real, external :: ranf_ 

contains

subroutine ocn_setterrain

!  Random noise

use ocn_vars, only: u,v,w,t
implicit none

integer i,j,k,m,it,jt,nx1,ny1,kb,kc,iter
real xxx,yyy,zzz, aaa, bbb, ccc
real tmp(nx,ny,nzm)

terra = 1.
terrau = 1.
terrav = 1.
terraw = 1.
k_terra = 1
k_terrau = 1
k_terrav = 1
kmax = 1

if(.not.doterrain) then
  elevation = 0.
  return
end if

if(nrestart.eq.0) then

  call task_rank_to_index(rank,it,jt)

  select case (terrain_type)

    case(0)

    case(1)

       elevation = -zi(nz)
       do j=1,ny
        do i=1,nx
         if(rank.eq.nsubdomains/2.and.i.le.5*nx/8.and.i.ge.3*nx/8 &
            .and.j.le.5*ny/8.and.j.ge.3*ny/8.) then
              elevation(i,j) = -30.
         end if
        end do
        write(*,'(16f7.1)') (elevation(i,j),i=1,nx)
       end do

    case default

       if(masterproc) print*,'terrain_type ',terrain_type, &
                     ' is not defined in setperturb(). Exitting...'
       call task_abort()

  end select

end if ! nrestart.eq.0

do k=1,nzm
  do j=1,ny
    do i=1,nx
        if(.not.(dooceanonly.and.readlandmask)) then
          if(elevation(i,j).ge.z(k)-zi(nz)) then 
             terra(i,j,k) = 0.
             kmax = max(kmax,k)
          end if
        else
          if(landmask(i,j).eq.1) then
             terra(i,j,k) = 0.
          end if
        end if
    end do
  end do
end do

tmp(1:nx,1:ny,1:nzm) = t(1:nx,1:ny,1:nzm)
t = terra
if(dompi) then
 call ocn_task_exchange(t,dimx1_s,dimx2_s,dimy1_s,dimy2_s,nzm,3+NADVS,3+NADVS,3+NADVS,3+NADVS,4)
else
 call ocn_bound_exchange(t,dimx1_s,dimx2_s,dimy1_s,dimy2_s,nzm,3+NADVS,3+NADVS,3+NADVS,3+NADVS,4)
endif
terra = t
t(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)

! repair terrain to avoid "one point holes":
do k=1,nzm
 do j=1,ny
  do i=1,nx
   if(terra(i,j,k).eq.1.and.terra(i-1,j,k)+terra(i+1,j,k)+terra(i,j-1,k)+terra(i,j+1,k).eq.0.) then
      terra(i,j,k) = 0. 
      kmax = max(kmax,k)
   end if 
  end do
 end do
end do

tmp(1:nx,1:ny,1:nzm) = t(1:nx,1:ny,1:nzm)
t = terra
if(dompi) then
 call ocn_task_exchange(t,dimx1_s,dimx2_s,dimy1_s,dimy2_s,nzm,3+NADVS,3+NADVS,3+NADVS,3+NADVS,4)
else
 call ocn_bound_exchange(t,dimx1_s,dimx2_s,dimy1_s,dimy2_s,nzm,3+NADVS,3+NADVS,3+NADVS,3+NADVS,4)
endif
terra = t
t(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)

if(dowallx) then
  if(mod(rank,nsubdomains_x).eq.0) then
    do k=1,nzm
     do j=dimy1_s,dimy2_s
       do i=dimx1_s,0
         terra(i,j,k) = terra(1,j,k)
       end do
     end do
    end do
  end if
  if(mod(rank,nsubdomains_x).eq.nsubdomains_x-1) then
    do k=1,nzm
     do j=dimy1_s,dimy2_s
       do i=nx+1,dimx2_s
         terra(i,j,k) = terra(nx,j,k)
       end do
     end do
    end do
  end if
end if

if(dowally) then
  if(rank.lt.nsubdomains_x) then
    do k=1,nzm
     do j=dimy1_s,0
       do i=dimx1_s,dimx2_s
         terra(i,j,k) = terra(i,1,k)
       end do
     end do
    end do
  end if
  if(rank.gt.nsubdomains-nsubdomains_x-1) then
    do k=1,nzm
     do j=ny+1,dimy2_s
       do i=dimx1_s,dimx2_s
         terra(i,j,k) = terra(i,ny,k)
       end do
     end do
    end do
  end if
end if

elevationg = zi(nz)
do k=1,nzm
  do j=1,ny
    do i=1,nx
        if(terra(i,j,k).lt.1.) then 
         elevationg(i,j) = zi(nz)-zi(k+1)
        end if
    end do
  end do
end do

do k=1,nzm
  do j=dimy1_s,dimy2_s
    do i=dimx1_s,dimx2_s
       if(terra(i,j,k).lt.1.) k_terra(i,j) = k+1
    end do
  end do
end do

if(dooceanonly.and..not.readlandmask) then
 do j=1,ny
  do i=1,nx
   if(k_terra(i,j).eq.nz) then
      landmask(i,j) = 1.
   else
      landmask(i,j) = 0.
   end if
  end do
 end do
end if

! check consistency of landmask and terrain (should be some depth to ocean)

do j=1,ny
 do i=1,nx
  if(landmask(i,j).ne.1.and.k_terra(i,j).eq.nz) then
   print*,'rank=',rank,' ocean has zero depth when landmask.ne.1 at i,j=', i,j
   call task_abort()
  end if
 end do
end do


do k=1,nzm
  do j=1,ny
    do i=1,nx
        if(terra(i,j,k).lt.1) then
         terraw(i,j,k) = 0
         terraw(i,j,k+1) = 0
        end if
    end do
  end do
end do


do k=1,nzm
  do j=1,ny
    do i=1,nx
       terrau(i,j,k) = min(terra(i-1,j,k),terra(i,j,k))
       terrav(i,j,k) = min(terra(i,j-YES3D,k),terra(i,j,k))
    end do
  end do
end do

call task_barrier()

if(dompi) then
 tmp(1:nx,1:ny,1:nzm) = u(1:nx,1:ny,1:nzm)
 u=terrau
 call ocn_task_exchange(u,dimx1_u,dimx2_u,dimy1_u,dimy2_u,nzm,2,3,2+NADV,2+NADV,1)
 terrau=u
 u(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)
 tmp(1:nx,1:ny,1:nzm) = v(1:nx,1:ny,1:nzm)
 v=terrav
 call ocn_task_exchange(v,dimx1_v,dimx2_v,dimy1_v,dimy2_v,nzm,2+NADV,2+NADV,2,3,2)
 terrav=v
 v(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)
 tmp(1:nx,1:ny,1:nzm) = w(1:nx,1:ny,1:nzm)
 w=terraw
 call ocn_task_exchange(w,dimx1_w,dimx2_w,dimy1_w,dimy2_w,nz,2+NADV,2+NADV,2+NADV,2+NADV,3)
 terraw=w
 w(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)
 w(1:nx,1:ny,nz) = 0.
else
 tmp(1:nx,1:ny,1:nzm) = u(1:nx,1:ny,1:nzm)
 u=terrau
 call ocn_bound_exchange(u,dimx1_u,dimx2_u,dimy1_u,dimy2_u,nzm,2,3,2+NADV,2+NADV,1)
 terrau=u
 u(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)
 tmp(1:nx,1:ny,1:nzm) = v(1:nx,1:ny,1:nzm)
 v=terrav
 call ocn_bound_exchange(v,dimx1_v,dimx2_v,dimy1_v,dimy2_v,nzm,2+NADV,2+NADV,2,3,2)
 terrav=v
 v(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)
 tmp(1:nx,1:ny,1:nzm) = w(1:nx,1:ny,1:nzm)
 w=terraw
 call ocn_bound_exchange(w,dimx1_w,dimx2_w,dimy1_w,dimy2_w,nz,2+NADV,2+NADV,2+NADV,2+NADV,3)
 terraw=w
 w(1:nx,1:ny,1:nzm) = tmp(1:nx,1:ny,1:nzm)
endif

call task_barrier()

call ocn_boundaries(1)
call ocn_boundaries(2)

do k=1,nzm,1
  do j=dimy1_u,dimy2_u
    do i=dimx1_u,dimx2_u
       if(terrau(i,j,k).lt.1.) k_terrau(i,j) = k+1
    end do
  end do
end do
do k=1,nzm,1
  do j=dimy1_v,dimy2_v
    do i=dimx1_v,dimx2_v
       if(terrav(i,j,k).lt.1.) k_terrav(i,j) = k+1
    end do
  end do
end do

!do k=1,nzm
! do j=1,ny
!  do i=1,nx
!    if(terra(i,j,k).eq.1.and.terrau(i+1,j,k)+terrau(i,j,k)+terrav(i,j+1,k)+terrav(i,j,k)+ &
!       terraw(i,j,k+1)+terraw(i,j,k).le.2) &
!       write(*,'(4i5,7f3.0)')rank,i,j,k,terra(i,j,k),terrau(i+1,j,k),terrau(i,j,k),terrav(i,j+1,k),terrav(i,j,k), terraw(i,j,k),terraw(i,j,k+1)    
!  end do
! end do
!end do




end subroutine ocn_setterrain

end module ocn_terrain
