
subroutine ocn_tke_full

!	this subroutine solves the TKE equation

use ocn_vars
use ocn_sgs
use ocn_params
use ocn_terrain, only: k_terra
use gsw_mod_kinds, only: r8
use gsw_mod_toolbox, only: gsw_rho_alpha_beta
implicit none

real def2(nx,ny,nzm)	
real grd,betdz,Ck,Ce,Ces,Ce1,Ce2,smix,Pr,Cee,Cs
real buoy_sgs,ratio,a_prod_sh,a_prod_bu,a_diss,smixs,fb,ustar,eta,smixmo
real lmo(nx,ny)
real(r8) alpha1, beta1, rho1
integer i,j,k,kc,kb

call t_startf('ocn_tke_full')

!Cs = 0.1944
Cs = 0.15
Ck=0.1
Ce=Ck**3/Cs**4
Ces=Ce/0.7*3.0	

if(RUN3D) then
  call ocn_shear_prod3D(def2)
else
  call ocn_shear_prod2D(def2)
endif

! determine monin-obukhov scale from total surface flux

do j=1,ny
 do i=1,nx
  call gsw_rho_alpha_beta(real(s(i,j,nzm),r8), real(t(i,j,nzm),r8), real(pres(nzm),r8),rho1,alpha1,beta1)  
  fb = ggr*(alpha1*fluxtt(i,j)-beta1*fluxts(i,j))  
  ustar = sqrt(sqrt(fluxtu(i,j)**2+fluxtv(i,j)**2))
  if(abs(fb).gt.1.e-9) then
    lmo(i,j) = -ustar**3/(0.4*fb)
  else
    lmo(i,j) = -sign(1.,fb)*1.e5
  end if
!  print*,'>>>',fluxtt(i,j),fluxts(i,j),fb,ustar,lmo(i,j)
 end do
end do

do k=1,nzm      
  kb=k-1
  kc=k+1

  grd=dz*adz(k)

  betdz=ggr/dz/(adzw(kc)+adzw(k))
  Ce1=Ce/0.7*0.19
  Ce2=Ce/0.7*0.51
  if(k.eq.1) then
    kb=1
    kc=2
    betdz=ggr/dz/adzw(kc)
    Ce1=Ces/0.7*0.19
    Ce2=Ces/0.7*0.51
  end if
  if(k.eq.nzm) then
    kb=nzm-1
    kc=nzm
    betdz=ggr/dz/adzw(k)
    Ce1=Ces/0.7*0.19
    Ce2=Ces/0.7*0.51
  end if
  tkelediss(k) = 0.
  tkesbdiss(k) = 0.
  tkesbshear(k)= 0.
  tkesbbuoy(k) = 0.
  do j=1,ny
  do i=1,nx
   if(k_terra(i,j).gt.nzm) cycle
!  SGS buoyancy flux

!bloss: removed temperature diagnostics for omn.
!         - use mass weighted qsat, dqsat and latent heat for cloud 
!         - separate buoyancy contributions for precipitating water and ice.
   
   
!   call gsw_rho_alpha_beta(real(s(i,j,k),r8), real(t(i,j,k),r8), real(pres(k),r8),rho1,alpha1,beta1)
!   buoy_sgs=betdz*(alpha1*(tabs(i,j,kc)-tabs(i,j,kb))-beta1*(s(i,j,kc)-s(i,j,kb)) )
   buoy_sgs=-0.001*betdz*(rhoo(i,j,kc)-rhoo(i,j,kb))

   if(buoy_sgs.le.0.) then
     smix=grd
   else
     smix=max(0.1*grd, sqrt(0.76*tk(i,j,k)/Ck/sqrt(buoy_sgs+1.e-10)))
   end if
   smixmo = 0.4*(zi(nz)-z(k))
   eta = (zi(nz)-z(k))/lmo(i,j)
   if(eta.lt.0.) then
     smixmo = smixmo*(1.-100*eta)**0.2
   end if
   smix = 1./(1./smix+1./smixmo)
!   print*,'>>',smix,smixmo

   ratio=smix/grd
   Pr=1. 
!   Pr=1. +2.*ratio
   Cee=Ce1+Ce2*ratio

   if(dosmagor) then

     tk(i,j,k)=sqrt(Ck**3/Cee*max(0.,def2(i,j,k)-Pr*buoy_sgs))*smix**2
     tke(i,j,k) = (tk(i,j,k)/(Ck*smix))**2
     a_prod_sh=(tk(i,j,k)+0.001)*def2(i,j,k)
     a_prod_bu=-(tk(i,j,k)+0.001)*Pr*buoy_sgs
     a_diss=a_prod_sh+a_prod_bu

   else

     tke(i,j,k)=max(0.,tke(i,j,k))
     a_prod_sh=(tk(i,j,k)+0.001)*def2(i,j,k)
     a_prod_bu=-(tk(i,j,k)+0.001)*Pr*buoy_sgs
     a_diss=min(tke(i,j,k)/(4.*dt),Cee/smix*tke(i,j,k)**1.5)!cap the diss rate (useful for large time steps
     tke(i,j,k)=max(0.,tke(i,j,k)+dtn*(max(0.,a_prod_sh+a_prod_bu)-a_diss))
     tk(i,j,k)=Ck*smix*sqrt(tke(i,j,k))

   end if
	
   tkh(i,j,k)=Pr*tk(i,j,k)

   tkelediss(k) = tkelediss(k) - a_prod_sh
   tkesbdiss(k) = tkesbdiss(k) + a_diss
   tkesbshear(k)= tkesbshear(k)+ a_prod_sh
   tkesbbuoy(k) = tkesbbuoy(k) + a_prod_bu

  end do ! i
  end do ! j

  tkelediss(k) = tkelediss(k)/float(nx*ny)


end do ! k

call t_stopf('ocn_tke_full')

end


