
subroutine ocn_averageXY(f,dimx1,dimx2,dimy1,dimy2,dimz,fm)

! average scalar field
use ocn_grid, only: nx,ny,nzm,wgt
use ocn_terrain, only: terra
implicit none
integer dimx1, dimx2, dimy1, dimy2, dimz
real f(dimx1:dimx2, dimy1:dimy2, dimz),fm(nzm) 
real(8) ff,factor
integer i,j,k	
factor = 1./dble(nx*ny)
do k =1,nzm
 ff = 0.
 do j =1,ny
  do i =1,nx
    ff = ff + f(i,j,k)*wgt(j,k)*terra(i,j,k)
  end do
 end do
 ff = ff*factor
 fm(k) = real(ff)
end do 
end


	
subroutine ocn_averageXY_MPI(f,dimx1,dimx2,dimy1,dimy2,dimz,fm)

! average scalar field
use ocn_grid, only: dompi,nsubdomains,nx,ny,nzm,wgt
use ocn_terrain, only: terra
implicit none
integer dimx1, dimx2, dimy1, dimy2, dimz
real f(dimx1:dimx2, dimy1:dimy2, dimz),fm(nzm)
real(8) fm1(nzm),fm2(nzm),factor
integer i,j,k
factor = 1./dble(nx*ny)
do k =1,nzm
 fm1(k) = 0.
 do j =1,ny
  do i =1,nx
    fm1(k) = fm1(k) + f(i,j,k)*wgt(j,k)*terra(i,j,k)
  end do
 end do
 fm1(k) = fm1(k) * factor
end do
if(dompi) then
 do k =1,nzm
   fm2(k) = fm1(k)
 end do
 call task_sum_real8(fm2,fm1,nzm)
 do k=1,nzm
  fm(k)=real(fm1(k)/dble(nsubdomains))
 end do
else
 do k=1,nzm
  fm(k)=real(fm1(k))
 end do
endif
end


		
	
	
subroutine ocn_fminmax_print(name,f,dimx1,dimx2,dimy1,dimy2,dimz)

use ocn_grid
implicit none
integer dimx1, dimx2, dimy1, dimy2, dimz
real f(dimx1:dimx2, dimy1:dimy2, dimz),fmn(nz),fmx(nz)
character *(*) name
real fmin(1),fmax(1),fff(1),fsum(1)
integer i,j,k
	
do k=1,dimz
 if(dimx2.eq.1.and.dimy2.eq.1) then
   fmn(k) = f(1,1,k)	
   fmx(k) = f(1,1,k)	
 else
   fmn(k) = 1.e30
   fmx(k) =-1.e30
   do j=1,ny
    do i=1,nx
     fmn(k) = min(fmn(k),f(i,j,k))
     fmx(k) = max(fmx(k),f(i,j,k))
    end do
   enddo
 end if
enddo
fmin(1) = 1.e30
fmax(1) =-1.e30
do k=1,dimz
 fmin(1) = min(fmin(1),fmn(k))
 fmax(1) = max(fmax(1),fmx(k))
end do
	
if(dompi) then
  fff(1)=fmax(1)
  call ocn_task_max_real(fff(1),fmax(1),1)
  fff(1)=fmin(1)
  call ocn_task_min_real(fff(1),fmin(1),1)
else
  fsum = sum(f)
end if
if(masterproc) print *,name,fmin,fmax !,fsum
end




subroutine ocn_fminmax1_print(name,f,n1,n2,n3)

use ocn_grid
implicit none
integer n1,n2,n3
real f(n1,n2,n3)
character *(*) name
real fmin(1),fmax(1),fff(1)

fmin(1) = minval(f)
fmax(1) = maxval(f)

if(dompi) then
  fff(1)=fmax(1)
  call ocn_task_max_real(fff(1),fmax(1),1)
  fff(1)=fmin(1)
  call ocn_task_min_real(fff(1),fmin(1),1)
end if
if(masterproc) print *,name,fmin,fmax
end

subroutine ocn_setvalue(f,n,f0)
implicit none
integer n
real f(n), f0
integer k
do k=1,n
 f(k)=f0
end do
end

	
!----------------------------------------
! average a global 3D array alone x direction
! with the result known to each subdomain

subroutine ocn_mean_x_3D(in, out)

use ocn_grid
implicit none
real, intent(in) ::  in(nx,ny,nzm)
real, intent(out) :: out(ny,nzm)
real, allocatable ::  buf(:,:,:)
integer i,j,k,m
integer n_in, count, reqs_in(max(1,nsubdomains_x-1)), rnk, tag
integer n_out, reqs_out(nsubdomains_x-1), tags(nsubdomains_x-1), ranks(nsubdomains_x-1)
logical flag(nsubdomains_x)

! average locally first:
do k=1,nzm
 do j=1,ny
  out(j,k) = sum(in(1:nx,j,k))
 end do
end do
out(:,:)=out(:,:)/dble(nx)

if(dompi) then

 if(mod(rank,nsubdomains_x).ne.0) then
   call ocn_task_bsend_float(rank-mod(rank,nsubdomains_x),out(1:ny,1:nzm),ny*nzm, 31)
   call ocn_task_breceive_float(out(1:ny,1:nzm),ny*nzm,rnk,tag)
 else
   allocate(buf(ny,nzm,max(1,nsubdomains_x-1)))
   n_in = 0
   do i=1,nsubdomains_x-1
     n_in = n_in + 1
     call ocn_task_receive_float(buf(1:ny,1:nzm,n_in),ny*nzm,reqs_in(n_in))
     flag(n_in) = .false.
   end do
   count = n_in
   do while (count .gt. 0)
    do m = 1,n_in
     if(.not.flag(m)) then
         call task_test(reqs_in(m), flag(m), rnk, tag)
         if(flag(m)) then
           count=count-1
           out(1:ny,1:nzm) = out(1:ny,1:nzm) + buf(1:ny,1:nzm,m)
         endif
     endif
    end do
   end do
   out = out/dble(nsubdomains_x)
   n_out = 0.
   do i=1,nsubdomains_x-1
     n_out=n_out+1
     call ocn_task_send_float(rank+i,out(1:ny,1:nzm),ny*nzm,31,reqs_out(n_out))
   end do
   call task_waitall(n_out,reqs_out,tags,ranks)
   deallocate(buf)
 end if

end if

call task_barrier()

end
!----------------------------------------
! average a global 2D array (x-y) alone x direction
! with the result known to each subdomain

subroutine ocn_mean_x_2D(in, out)

use ocn_grid
implicit none
real, intent(in) ::  in(nx,ny)
real, intent(out) :: out(ny)
real buf(ny,max(1,nsubdomains_x-1))
integer i,j,k,m
integer n_in, count, reqs_in(max(1,nsubdomains_x-1)), rnk, tag
integer n_out, reqs_out(nsubdomains_x-1), tags(nsubdomains_x-1), ranks(nsubdomains_x-1)
logical flag(nsubdomains_x)

! average locally first:
do j=1,ny
 out(j) = sum(in(1:nx,j))/dble(nx)
end do

if(dompi) then

 if(mod(rank,nsubdomains_x).ne.0) then
   call ocn_task_bsend_float(rank-mod(rank,nsubdomains_x),out(1:ny),ny, 31)
   call ocn_task_breceive_float(out(1:ny),ny,rnk,tag)
 else
   n_in = 0
   do i=1,nsubdomains_x-1
     n_in = n_in + 1
     call ocn_task_receive_float(buf(1:ny,n_in),ny,reqs_in(n_in))
     flag(n_in) = .false.
   end do
   count = n_in
   do while (count .gt. 0)
    do m = 1,n_in
     if(.not.flag(m)) then
         call task_test(reqs_in(m), flag(m), rnk, tag)
         if(flag(m)) then
           count=count-1
           out(1:ny) = out(1:ny) + buf(1:ny,m)
         endif
     endif
    end do
   end do
   out = out/dble(nsubdomains_x)
   n_out=0.
   do i=1,nsubdomains_x-1
     n_out = n_out+1
     call ocn_task_send_float(rank+i,out(1:ny),ny,31,reqs_out(n_out))
   end do
   call task_waitall(n_out,reqs_out,tags,ranks)
 end if

end if

call task_barrier()
end

subroutine zonal_avg2D(fld)

use ocn_terrain, only: terra
use ocn_grid
implicit none
real fld(nx,ny)
real tmp(nx,ny),tmp1(ny)
integer i,j
do j=1,ny
 do i=1,nx
   tmp(i,j) = fld(i,j)*wgtx(j,nzm)*terra(i,j,nzm)
 end do
end do
call ocn_mean_x_2D(tmp,tmp1)
do j=1,ny
 do i=1,nx
   fld(i,j) = tmp1(j)
 end do
end do

end subroutine 


subroutine ocn_readsurface(filename,field,dayobs)

! read surface data 

use ocn_grid
use params, only: docyclic
implicit none
character*(*), intent(in) ::  filename
real, intent(out) ::  field(nx,ny,2)   ! two time saples if available
real, intent(out) :: dayobs(2) ! days corresponding to field samples

integer i,nn,it,jt,nobs,nx1,ny1
real(4), allocatable ::  fld(:,:)
real(4), allocatable :: days(:)
real coefi, day1

call task_rank_to_index(rank,it,jt)
if(masterproc) print*,'reading data from ',trim(filename)
open(11,file=trim(filename),status='old',form='unformatted')
read(11) nobs
if(masterproc) print*,'nobs=',nobs
read(11) nx1
if(masterproc) print*,'nx (obs)=',nx1
read(11) ny1
if(masterproc) print*,'ny (obs)=',ny1
if(nx1.ne.nx_gl.or.ny1.ne.ny_gl) then
  if(masterproc) print*,'dimensions of domain in file '//trim(filename)//' is' // &
              'different from model domain size: nx=',nx1, &
              'ny=',ny1,'  Quitting...'
  call task_abort()
end if
allocate (days(1:nobs))
read(11) days(1:nobs)
if(masterproc) print*,'days=',days
if(docyclic) then
  day1=day-int(day)+mod(int(day),365)
else
  day1 = day
end if
nn=1
if(docyclic.and.(days(nobs).gt.365..or.days(1).lt.0.)) then
  print*,'error reading file ',filename,'docyclic=',docyclic,'time in should not exceed 365 days'
  call task_abort
end if
if(.not.docyclic.and.nobs.gt.1.and.(day1.gt.days(nobs).or.day1.lt.days(1))) then
 if(masterproc) print*,'error reading file ',filename, 'nobs=',nobs,' day1=',day1, &
               ' days(1)=',days(1),' days(nobs)=',days(nobs),'docyclic=',docyclic
 call task_abort
end if
allocate(fld(1:nx_gl,1:ny_gl))
if(nobs.gt.1.and.docyclic.and.(day1.le.days(1).or.day1.gt.days(nobs))) then
  read(11) fld(1:nx_gl,1:ny_gl)
  field(1:nx,1:ny,2) = fld(1+it:nx+it,1+jt:ny+jt)
  do i=1,nobs-2
    read(11)
  end do
  read(11) fld(1:nx_gl,1:ny_gl)
  field(1:nx,1:ny,1) = fld(1+it:nx+it,1+jt:ny+jt)
  if(day1.le.days(1)) then
    dayobs(2) = days(1)+day-day1
    dayobs(1) = days(nobs)+day-day1-365.
  else
    dayobs(2) = days(1)+day-day1+365.
    dayobs(1) = days(nobs)+day-day1
  end if
else
  do i=1,nobs-1
   if(day1.gt.days(i)) then
     nn=i
   endif
  end do
  do i=1,nn-1
     read(11)
  end do
  field = 0.
  read(11) fld(1:nx_gl,1:ny_gl)
  field(1:nx,1:ny,1) = fld(1+it:nx+it,1+jt:ny+jt)
  dayobs(1) = days(nn)+day-day1
  dayobs(2) = 0.
  if(nobs.gt.1) then
   read(11) fld(1:nx_gl,1:ny_gl)
   field(1:nx,1:ny,2) = fld(1+it:nx+it,1+jt:ny+jt)
   dayobs(2) = days(nn+1)+day-day1
  end if
end if
close (11)
if(masterproc) print*,'min/max:',minval(fld),maxval(fld)
deallocate(fld)
if(nobs.gt.1.and.masterproc) print*,'day1 nobs days(1) and days(nobs)',day1, nobs, days(1),days(nobs), &
                                    'dayobs(1)=',dayobs(1),'dayobs(2)=',dayobs(2)
deallocate(days)
end


subroutine ocn_write_field1D (ntape,f,z,nlevs)

! write 1D arrays into a general
! (not dependent on number of PEs) restart file

use ocn_grid, only: masterproc
implicit none
! Input:

integer, intent(in) :: ntape
real,intent(in) ::  f(nlevs)
real,intent(in) ::  z(nlevs)
integer, intent(in) ::  nlevs
integer k

! Local:

if(masterproc) write(ntape) nlevs
if(masterproc) write(ntape) z(1:nlevs)
do k=1,nlevs
   if(masterproc) write(ntape) f(k)
end do

end


subroutine ocn_write_field2D (ntape,f)

! write full global arrays into a general
! (not dependent on number of PEs) restart file

use ocn_grid, only: masterproc, nx, ny, nx_gl, ny_gl, rank, nsubdomains
implicit none
! Input:

integer, intent(in) :: ntape
real,intent(in) ::  f(nx,ny)

! Local:

real(4), allocatable :: fld(:,:)
real(4) buf(nx,ny)
integer irank,ranksource,tagsource,ii,jj,req

if(masterproc) then
  allocate(fld(nx_gl,ny_gl))
  fld(1:nx,1:ny) = f(:,:)
end if
do irank = 1, nsubdomains-1
    call task_barrier()
    if(irank.eq.rank) then
      call ocn_task_bsend_float(0,f(:,:),nx*ny,irank)
    end if
    if(masterproc) then
      call ocn_task_receive_float(buf(:,:),nx*ny,req)
      call task_wait(req,ranksource,tagsource)
      call task_rank_to_index(ranksource,ii,jj)
      fld(ii+1:ii+nx,jj+1:jj+ny) = buf(:,:)
    end if
end do
if(masterproc) then
  write(ntape) fld
  deallocate(fld)
end if

end


subroutine ocn_write_field3D (ntape,f,z,nlevs)

! write full global arrays into a general
! (not dependent on number of PEs) restart file

use ocn_grid, only: masterproc, nx, ny, nx_gl, ny_gl, rank, nsubdomains
implicit none
! Input:

integer, intent(in) :: ntape
real,intent(in) ::  f(nx,ny,nlevs)
real,intent(in) ::  z(nlevs)
integer, intent(in) ::  nlevs

! Local:

real(4), allocatable ::  fld(:,:)
real(4) buf(nx,ny)
integer irank,ranksource,tagsource,ii,jj,k,req

if(masterproc) write(ntape) nlevs
if(masterproc) write(ntape) z(1:nlevs)
do k=1,nlevs
   if(masterproc) then
     allocate(fld(1:nx_gl,1:ny_gl))
     fld(1:nx,1:ny) = f(:,:,k)
   end if
   do irank = 1, nsubdomains-1
       call task_barrier()
       if(irank.eq.rank) then
         call ocn_task_bsend_float(0,f(:,:,k),nx*ny,irank)
       end if
       if(masterproc) then
         call ocn_task_receive_float(buf(:,:),nx*ny,req)
         call task_wait(req,ranksource,tagsource)
         call task_rank_to_index(ranksource,ii,jj)
         fld(ii+1:ii+nx,jj+1:jj+ny) = buf(:,:)
       end if
   end do
   if(masterproc) then
     write(ntape) fld
     deallocate(fld)
   end if
end do

end

subroutine ocn_read_field1D (ntape,f,z,nlevs)

! read arrays from a general
! (not dependent on number of PEs) restart file
! with bilinear interpolation

use ocn_grid, only: masterproc
implicit none
! Input:
integer ntape
real, intent(out) ::  f(nlevs)
integer, intent(in) :: nlevs ! numver of layers in new array
real, intent(in) :: z(nlevs) ! height of new array's levels
! Local:
real(4) fld(2)
real(4),allocatable ::  zr(:)
real wgt
integer k1,k2,k3,kk
integer nlevs1
integer k
read(ntape) nlevs1
allocate(zr(nlevs1))
read(ntape) zr(1:nlevs1)
!if(z(nlevs).gt.zr(nlevs1)) then
!  if(masterproc) then
!       print*,'Restart_read_field1D: error: z(nlevs).gt.zr(nlevs1)! ', &
!       'nlevs=',nlevs,' nlevs1=',nlevs1,' z(nlevs)=',z(nlevs),' zr(nlevs1)=',zr(nlevs1), &
!       ' Quitting...'
!  end if
!  call task_abort
!end if
k1=1
k2=2
read(ntape) fld(k1)
read(ntape) fld(k2)
kk=1
do k=1,nlevs
   do while(kk.lt.nlevs1-1.and.z(k).gt.zr(kk+1)) 
       k3=k1
       k1=k2
       k2=k3
       kk=kk+1
       read(ntape) fld(k2)
   end do
   wgt = (z(k)-zr(kk))/(zr(kk+1)-zr(kk))
   f(k) =  (1.-wgt)*fld(k1)+wgt*fld(k2)
end do ! k
do k=kk+2,nlevs1
  read(ntape)
end do

deallocate(zr)
end


subroutine ocn_read_field2D (ntape,f,nxr,nyr)

! read 2D arrays from a general
! (not dependent on number of PEs) restart file
! with bilinear interpolation

use ocn_grid, only: nx, ny, nx_gl, ny_gl, rank, masterproc
implicit none
! Input:
integer ntape
real, intent(out) ::  f(nx,ny)
integer, intent(in) :: nxr,nyr ! size of arrays in restart file
! Local:
real(4),allocatable ::  fld(:,:)
real dx, dy, x, y
integer m,n,m1,n1
integer ii,jj,iii,jjj
integer i,j
call task_rank_to_index(rank,iii,jjj)
allocate(fld(nxr,nyr))
ii=nx_gl/nxr
jj=ny_gl/nyr
dx=real(nxr)/nx_gl
dy=real(nyr)/ny_gl
read(ntape) fld(1:nxr,1:nyr)
do j=1,ny
 n = (j-1)*dy+jjj/jj+1
 y = (j-1)*dy-(n-jjj/jj-1)
 n1=n+1
 if(n1.gt.nyr) n1=1
 do i=1,nx
  m = (i-1)*dx+iii/ii+1
   x = (i-1)*dx-(m-iii/ii-1)
   m1=m+1
   if(m1.gt.nxr) m1=1
   f(i,j) = (1.-x)*(1.-y)*fld(m,n)+x*(1.-y)*fld(m1,n)+ &
                             (1.-x)*y*fld(m,n1)+x*y*fld(m1,n1) 
 end do ! i
end do ! j

deallocate(fld)
end


subroutine ocn_read_field3D (ntape,f,nxr,nyr,z,nlevs)

! read arrays from a general
! (not dependent on number of PEs) restart file
! with bilinear interpolation

use ocn_grid, only: nx, ny, nx_gl, ny_gl, rank, masterproc
implicit none
! Input:
integer ntape
real, intent(out) ::  f(nx,ny,nlevs)
integer, intent(in) :: nxr,nyr ! size of arrays in restart file
integer, intent(in) :: nlevs ! numver of layers in new array
real, intent(in) :: z(nlevs) ! height of new array's levels
! Local:
real(4),allocatable ::  fld(:,:,:)
real(4),allocatable ::  zr(:)
real dx, dy, x, y, wgt
integer m,n,m1,n1,k1,k2,k3,kk
integer nlevs1,ii,jj,iii,jjj
integer k,i,j
call task_rank_to_index(rank,iii,jjj)
read(ntape) nlevs1
if(masterproc) print*,'>>>>> nlevs1',nlevs1
if(nlevs1.eq.0) then
! interpolate in the vertical
  read(ntape) nlevs1
  allocate(zr(nlevs1))
  read(ntape) zr(1:nlevs1)
  if(masterproc) print*,'>>>>> zr',zr
  !if(z(nlevs).gt.zr(nlevs1)) then
  !  if(masterproc) then
  !       print*,'Restart_read_field3D: error: z(nlevs).gt.zr(nlevs1)! ', &
  !       'nlevs=',nlevs,' nlevs1=',nlevs1,' z(nlevs)=',z(nlevs),' zr(nlevs1)=',zr(nlevs1), &
  !       ' Quitting...'
  !  end if
  !  call task_abort
  !end if
  allocate(fld(nxr,nyr,2))
  ii=nx_gl/nxr
  jj=ny_gl/nyr
  dx=real(nxr)/nx_gl
  dy=real(nyr)/ny_gl
  k1=1
  k2=2
  read(ntape) fld(1:nxr,1:nyr,k1)
  read(ntape) fld(1:nxr,1:nyr,k2)
  kk=1
  do k=1,nlevs
     do while(kk.lt.nlevs1-1.and.z(k).gt.zr(kk+1))
         k3=k1
         k1=k2
         k2=k3
         kk=kk+1
         read(ntape) fld(1:nxr,1:nyr,k2)
     end do
     wgt = (z(k)-zr(kk))/(zr(kk+1)-zr(kk))
     do j=1,ny
       n = (j-1)*dy+jjj/jj+1
       y = (j-1)*dy-(n-jjj/jj-1)
       n1=n+1
       if(n1.gt.nyr) n1=1
       do i=1,nx
         m = (i-1)*dx+iii/ii+1
         x = (i-1)*dx-(m-iii/ii-1)
         m1=m+1
         if(m1.gt.nxr) m1=1
         f(i,j,k) = (1.-wgt)*((1.-x)*(1.-y)*fld(m,n,k1)+x*(1.-y)*fld(m1,n,k1)+ &
                           (1.-x)*y*fld(m,n1,k1)+x*y*fld(m1,n1,k1)) &
                    + wgt*((1.-x)*(1.-y)*fld(m,n,k2)+x*(1.-y)*fld(m1,n,k2)+ &
                           (1.-x)*y*fld(m,n1,k2)+x*y*fld(m1,n1,k2))
       end do ! i
      end do ! j
  end do ! k
  do k=kk+2,nlevs1
    read(ntape)
  end do
  deallocate(fld)
  deallocate(zr)
else
! dont interpolate in the vertical
  if(nlevs.gt.nlevs1) then
    print*,'Restart_read_field3D: error: nlevs.gt.nlevs1. nlevs=',nlevs,' nlevs1=',nlevs1,' quitting...'
    call task_abort
  end if
  allocate(fld(nxr,nyr,1))
  ii=nx_gl/nxr
  jj=ny_gl/nyr
  dx=real(nxr)/nx_gl
  dy=real(nyr)/ny_gl
  do k=1,nlevs1
   read(ntape) fld(1:nxr,1:nyr,1)
   do j=1,ny
     n = (j-1)*dy+jjj/jj+1
     y = (j-1)*dy-(n-jjj/jj-1)
     n1=n+1
     if(n1.gt.nyr) n1=1
     do i=1,nx
       m = (i-1)*dx+iii/ii+1
       x = (i-1)*dx-(m-iii/ii-1)
       m1=m+1
       if(m1.gt.nxr) m1=1
       f(i,j,k) = (1.-x)*(1.-y)*fld(m,n,1)+x*(1.-y)*fld(m1,n,1)+ &
                              (1.-x)*y*fld(m,n1,1)+x*y*fld(m1,n1,1)
     end do ! i
    end do ! j
   end do
   deallocate(fld)
end if

end


subroutine ocn_readlevels(filename,field)

! read layered data

use ocn_grid
implicit none
character*(*), intent(in) ::  filename
real, intent(out) ::  field(nx,ny,nzm)   ! output field

integer i,k,nn,nobs,nx1,ny1,nz1
real fld(nx,ny,nzm)
real(4), allocatable :: days(:)
real coef, dayy

if(masterproc) print*,'reading data from ',trim(filename)
open(11,file=trim(filename),status='old',form='unformatted')
read(11) nobs
read(11) nx1
read(11) ny1
read(11) nz1
if(masterproc)print*,'nx ny nz:',nx1,ny1,nz1
allocate (days(nobs))
read(11) days(1:nobs)
nn=1
do i=1,nobs-1
 if(day.gt.days(i)) then
   nn=i
 endif
end do
do i=1,nn-1
 read(11)
 do k=1,nz1
   read(11)
 end do
end do
call ocn_read_field3D(11,field,nx1,ny1,z,nzm)
dayy = day
if(nobs.gt.1) then
 call ocn_read_field3D(11,fld,nx1,ny1,z,nzm)
 do while(dayy.gt.365.)
  dayy = dayy-365.
 end do
 if(masterproc) print*,'interpolating: day=',dayy,'day1=',days(nn),'day2=',days(nn+1)
 coef=(dayy-days(nn))/(days(nn+1)-days(nn))
 field(:,:,:) = field(:,:,:)+coef*(fld(:,:,:)-field(:,:,:))
end if
close (11)
if(masterproc) print*,'min/max:',minval(field),maxval(field)
deallocate(days)
end


