#include "./../fppmacros"
     
subroutine ocn_write_fields2D
	
use ocn_vars
use ocn_params
use slm_vars, only: seaicemask
use vars, only: landmask
use grid, only: date_pr, datechar, lonu_gl
use pnetcdf_stuff, only: open_file2D_pnetcdf, close_file_pnetcdf, dopnetcdf

implicit none

character(200)filename
character(80) long_name
character(8)  name
character(10) timechar
character(3)  filetype
character(10) units
character(9) fileaction

integer i,j,k,nfields,nfields1,nsteplast
real(4) tmp(nx,ny,nzm)
real coef
logical flag


nfields= 31
if(doterrain) nfields=nfields+1
!===================================================================

nfields1=0

if(.not.dopnetcdf) then

 if(masterproc) then

  write(timechar,'(i10)') nstep*ncallocean
  do i=1,11-lenstr(timechar)-1
    timechar(i:i)='0'
  end do

! Make sure that the new run doesn't overwrite the file from the old run

    filetype = '.2D'

    if(save2Dsep) then
       fileaction = 'WRITE'
       filename='./OUT_2D/'//trim(case)//'_'//trim(caseid)//'_'// &
          trim(date_pr)//'_'//timechar(1:10)//'.OCN'//filetype
    else
       fileaction = 'READWRITE'
       filename='./OUT_2D/'//trim(case)//'_'//trim(caseid)//'.OCN'//filetype
    end if
    open(46,file=filename,status='unknown', &
                  form='unformatted',BUFFEREDYES ACTION=trim(fileaction))
    print*, 'Writting to file: '//trim(filename)
    if(.not.save2Dsep) then
       do while(.true.)
         read(46,end=222)  nsteplast
         if(nsteplast.ge.nstep) then
           backspace(46)
           goto 222   ! yeh, I know, it's bad ....
         end if
         read(46)
         read(46)
         read(46)
         read(46)
         read(46)
         do i=1,nfields
           read(46)
           read(46) flag
           if(.not.flag) read(46)
           read(46)
         end do
       end do
 222   continue
       print*,'nsteplast=',nsteplast
    end if

    write(46) nstep, dolatlon, time, datechar
    write(46) 'atm'
    write(46) nx,ny,nzm,nsubdomains, nsubdomains_x,nsubdomains_y,nfields
    write(46) real(dx,4),lat_gl,latv_gl,real(y_gl,4),real(yv_gl,4)
    write(46) real(dy,4),lon_gl,lonu_gl
    write(46) dble(nstep)*dt/(3600._8*24._8)+day0

 end if! masterproc

else

! open netcdf file:

 write(timechar,'(i10)') nstep
 do k=1,11-lenstr(timechar)-1
   timechar(k:k)='0'
 end do

 filename ='./OUT_2D/'//trim(case)//'_'//trim(caseid)//'_'// &
       trim(date_pr)//'_'//timechar(1:10)//'.2D_atm.nc'

 call open_file2D_pnetcdf(filename)

 if(masterproc) print*, 'Writting to file: '//trim(filename)

end if ! .not.dopnetcdf


if(.not.nstep.eq.1.and.save2Davg) then
   coef = 1./float(nsave2D)
else
   coef = 1.
end if


! 2D fields:

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=prec_xy(i,j)*coef*86400.
     end do
   end do
  name='PREC'
  long_name='Precipitation'
  units='mm/d'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=roff_xy(i,j)*coef*86400.
     end do
   end do
  name='ROFF'
  long_name='river runoff'
  units='mm/d'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=lhf_xy(i,j)*coef
     end do
   end do
  name='LHF'
  long_name='Latent Heat Flux'
  units='W/m2'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=shf_xy(i,j)*coef
     end do
   end do
  name='SHF'
  long_name='Sensible Heat Flux'
  units='W/m2'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=lhf_xy(i,j)*coef/lcond*86400.
     end do
   end do
  name='EVAP'
  long_name='Surface Evaporation'
  units='mm/d'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=-lhf_xy(i,j)*coef/lcond*86400. +prec_xy(i,j)*coef*86400.
     end do
   end do
  name='P-E'
  long_name='Precip - Evap'
  units='mm/d'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=taux_xy(i,j)*coef
     end do
   end do
  name='TAUX'
  long_name='Surface Stress in x'
  units='N/m2'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=tauy_xy(i,j)*coef
     end do
   end do
  name='TAUY'
  long_name='Surface Stress in y'
  units='N/m2'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)
  
   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=swns_xy(i,j)*coef
     end do
   end do
  name='SWNS'
  long_name='Surface Solar Net Flux'
  units='W/m2'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)
  
   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=lwns_xy(i,j)*coef
     end do
   end do
  name='LWNS'
  long_name='Surface Thermal Net Flux'
  units='W/m2'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=uatm_xy(i,j)*coef
     end do
   end do
  name='UATM'
  long_name='U above the surface'
  units='m/s'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=vatm_xy(i,j)*coef
     end do
   end do
  name='VATM'
  long_name='V above the surface'
  units='m/s'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=tatm_xy(i,j)*coef - 273.16
     end do
   end do
  name='TATM'
  long_name='T above the surface'
  units='deg C'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=qatm_xy(i,j)*coef*1.e3
     end do
   end do
  name='QATM'
  long_name='Q above the surface'
  units='g/kg'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)


if(doterrain) then
   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=elevationg(i,j)
     end do
   end do
  name='ZSFC'
  long_name='Surface Elevation (on grid)'
  units='m'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)
end if

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=usfc_xy(i,j)*coef 
     end do
   end do
  name='USFC'
  long_name='U at the surface'
  units='m/s'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=tsfc_xy(i,j)*coef 
     end do
   end do
  name='TSFC'
  long_name='surface Temperature'
  units='C'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=ssfc_xy(i,j)*coef 
     end do
   end do
  name='SSFC'
  long_name='surface alinity'
  units='g/kg'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=seaicefrac_xy(i,j)*coef
     end do
   end do
  name='SEAICE'
  long_name='seaicemask'
  units=' '
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=seaicethic_xy(i,j)*coef
     end do
   end do
  name='SEAICE_H'
  long_name='seaice thickness'
  units=' '
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
          tmp(i,j,1) = basins(i,j)
     end do
   end do
  name='BASIN'
  long_name='Basins'
  units=' '
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       if(landmask(i,j).eq.0) then
          tmp(i,j,1) = 1.
       else
          tmp(i,j,1) = 0.
       end if
     end do
   end do
  name='OCN'
  long_name='oceanmask'
  units=' '
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=u100_xy(i,j)*coef 
     end do
   end do
  name='U200'
  long_name='U at 100 m'
  units='m/s'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
      tmp(i,j,1)=vsfc_xy(i,j)*coef 
     end do
   end do
  name='VSFC'
  long_name='V at the surface'
  units='m/s'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=v100_xy(i,j)*coef 
     end do
   end do
  name='V200'
  long_name='V at 100 m'
  units='m/s'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
     do i=1,nx
       tmp(i,j,1)=w100_xy(i,j)*coef
     end do
   end do
  name='W500'
  long_name='W at 100 mb'
  units='m/s'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

  nfields1=nfields1+1
  do j=1,ny
    do i=1,nx
      tmp(i,j,1)= sst_min_xy(i,j)-273.16
    end do
  end do
  name='TMIN'
  long_name='Minumum Surface Temperature'
  units='C'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
    do i=1,nx
      tmp(i,j,1)= sst_max_xy(i,j)-273.16
    end do
   end do
  name='TMAX'
  long_name='Maximum Surface Temperature'
  units='C'
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
    do i=1,nx
      tmp(i,j,1)= cflz_max_xy(i,j)
    end do
   end do
  name='CFLZ'
  long_name='Maximum CFL in Z'
  units=''
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
    do i=1,nx
      tmp(i,j,1)= cflh_max_xy(i,j)
    end do
   end do
  name='CFLH'
  long_name='Maximum CFL in xy'
  units=''
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
    do i=1,nx
      tmp(i,j,1)= zcflz_max_xy(i,j)
    end do
   end do
  name='ZCFLZ'
  long_name='Height of Maximum CFL in Z'
  units=''
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

   nfields1=nfields1+1
   do j=1,ny
    do i=1,nx
      tmp(i,j,1)= zcflh_max_xy(i,j)
    end do
   end do
  name='ZCFLH'
  long_name='Height of Maximum CFL in xy'
  units=''
  call compress2D(tmp,nx,ny,name,long_name,units, &
                               save2Dbin,dompi,rank,nsubdomains)

call task_barrier()

!===================================================================


if(nfields.ne.nfields1) then
  if(masterproc) print*,'ocn_write_fields2D: error in nfields!!',' nfields=',nfields,'nfields1=',nfields1
  call task_abort()
end if


if(dopnetcdf) then
     call close_file_pnetcdf()
     dopnetcdf = .false.
else
  if(masterproc) then
     close(46)
     if(save2Dsep.and.dogzip2D) call systemf('gzip -f '//filename)
  endif
end if
if(masterproc) print*, 'Done. ', nfields1,'fields'


end
